//-----------------------------------------------------------------------------
//
// Phoenix
// Copyright (C) Microsoft Corporation.  All Rights Reserved.
//
// $Header: /Common/Columbia.Utilities/ReflectionHelpers.cs 2     9/29/06 6:20p Eaddy $
//
// Description:
//
//    .NET Reflection helper functions
//
// Remarks:
//
// ISSUE-TODO-t-marcea-2005/07/23-Phx#? Remove Reflection
// for unmanaged implementation
//
//-----------------------------------------------------------------------------
/**
 * $Log: /Common/Columbia.Utilities/ReflectionHelpers.cs $
 * 
 * 2     9/29/06 6:20p Eaddy
 * o Annotated program elements with the concern they realize
 * 
 * 1     5/02/06 7:41p Eaddy
 * 
 * 4     4/02/06 4:34p Asv2105
 * Added ability for BreakpointWeaver to pass MethodInfo to the
 * coordinator
 * 
 * Adjusted ICoordinator, IEventListener interfaces to match reality
 * 
 * Added ability for Coordinator to call static methods when a breakpoint
 * fires and pass off to EventMgr for non-static methods
 * 
 * 3     3/15/06 1:18p Eaddy
 * o Fixed bug where we failed to load aspect assemblies that
 * were not in the current directory
 * 
 * 2     2/24/06 8:27p Eaddy
 * o Added checkin comment
 */

using Phx.Aop.Attributes;

using System;
using System.IO;
using System.Collections.Generic;

using Dbg = System.Diagnostics.Debug;

namespace Columbia.Utilities
{
    /// <summary>
    /// .NET Reflection helper functions
    /// </summary>
    /// <remarks>
    /// ISSUE-TODO-t-marcea-2005/07/23-Phx#? Remove Reflection
    /// for unmanaged implementation
    /// </remarks>

    [Feature("Reflection")]
    public class ReflectionHelpers
    {
        /// <summary>
        /// Obtains the Type for a fully qualified type name by looking
        /// in all the loaded assemblies
        /// </summary>
        /// <remarks>
        /// Type.GetType(string) is supposed to do this exact thing
        /// but it doesn't work
        /// </remarks>
        /// <param name="typeName">Fully qualified name of the type
        /// (System.String)</param>
        /// <returns>Requested Type or null</returns>

        static public Type
        MyGetType
        (
            string typeName
        )
        {
            // ISSUE-TODO-t-marcea-2005/07/23-Phx#? Remove Reflection
            // for unmanaged implementation

            System.Type clrType = System.Type.GetType(typeName);
            if (clrType != null)
                return clrType;

            // Scour the other loaded assemblies for the type. This is
            // exactly what Type.GetType() is supposed to do (IMHO).

            System.AppDomain clrDomain = System.AppDomain.CurrentDomain;
            System.Reflection.Assembly[] clrAssemblies = clrDomain.GetAssemblies();
            foreach (System.Reflection.Assembly clrAssembly in clrAssemblies)
            {
                System.Reflection.Module[] clrModules = clrAssembly.GetModules();
                foreach (System.Reflection.Module clrModule in clrModules)
                {
                    clrType = clrModule.GetType(typeName);
                    if (clrType != null)
                        return clrType;
                }
            }

            return null;
        }

        /// <summary>
        /// Load an assembly into the CLR
        /// </summary>
        /// <param name="assemblyString">
        /// Can be an assembly short name (assembly name without
        /// an extension), relative path, or full path.
        /// </param>
        /// <returns>
        /// Loaded CLR assembly or null if the assembly couldn't be loaded
        /// </returns>

        public static System.Reflection.Assembly
        LoadAssembly
        (
            string assemblyString
        )
        {
            // ISSUE-TODO-t-marcea-2005/07/23-Phx#? Remove Reflection
            // for unmanaged implementation

            // Strip off the extension if present

            if (System.IO.Path.IsPathRooted(assemblyString))
            {
                return System.Reflection.Assembly.LoadFrom(assemblyString);
            }
            else
            {
                switch (Path.GetExtension(assemblyString))
                {
                    // ISSUE-TODO-t-marcea-2005/07/23-Load() and LoadFile()
                    // don't work if an extension is specified (verify?).
                    // However, the cases below probably don't cover all
                    // possibilities.

                    case ".dll":
                    case ".exe":
                    case ".netmodule":
                        assemblyString = Path.Combine(
                           Path.GetDirectoryName(assemblyString),
                           Path.GetFileNameWithoutExtension(assemblyString));
                        break;
                }

                return System.Reflection.Assembly.Load(assemblyString);
            }
        }
    }
}
