///////////////////////////////////////////////////////////////////////
//  Copyright (C) 2004 Columbia University.  All Rights Reserved.
//
//      $Header: /Common/Columbia.Utilities/XmlSerializerHelper.cs 2     9/29/06 6:20p Eaddy $
//
//    File name:	MyUtility.cs
//   Created By:	John Waugh
//   Created on:	4/22/05
//
//      Purpose:	
//
//  Description:	
//
///////////////////////////////////////////////////////////////////////
/**
 * $Log: /Common/Columbia.Utilities/XmlSerializerHelper.cs $
 * 
 * 2     9/29/06 6:20p Eaddy
 * o Annotated program elements with the concern they realize
 * 
 * 1     5/02/06 11:02a Eaddy
 * o Moved MyTrace into separate library
 * 
 * 5     5/28/05 1:21p Eaddy
 * Changed copyright
 * 
 * 4     4/22/05 5:14p Eaddy
 * o Added comment header
 */

using Phx.Aop.Attributes;

using System;
using System.Xml;
using System.Xml.Serialization;
using System.Reflection;
using System.Text;

namespace Columbia.Utilities
{
	[Feature("Persistence.Saving")]
	public interface ICustomSerialization
	{
		void Serialize(XmlWriter writer, Type[] extraTypes);
	}
	//custom Deserialization is done by implementing a function with the following signature:
	//public static object Deserialize(XmlReader reader, Type[] extraTypes)
	//(can't be done with an interface because it has to be static)
	[Feature("Persistence.Loading")]
	public interface IInitAfterDeserialization
	{
		void InitAfterDeserialization();
	}
	/// <summary>
	/// Use this class' static functions to serialize/deserialize Goblin objects
	/// This adds a layer of functionality on top of System.Xml.Serialization
	///		Namely, it allows an object to take part in its own serialization/deserialization
	///		by implementing one or more of
	///			ICustomSerialization, IInitAfterDeserialization
	///		If an object wishes to provide its own deserialization, its Deserialize method must by necessity
	///		be static
	///			Therefore, provide the following function in your class and XmlSerializerHelper will call it
	///			in lieu of its own deserialization code
	///				public static object Deserialize(XmlReader reader, Type[] extraTypes)
	/// </summary>
	[Feature("Persistence")]
	public class XmlSerializerHelper
	{
		[Feature("Persistence.Saving")]
		static public void Serialize(string filename, object o)
		{
			Serialize(filename,o,new Type[]{});
		}
		[Feature("Persistence.Saving")]
		static public void Serialize(XmlWriter writer, object o)
		{
			Serialize(writer,o,new Type[]{});
		}
		[Feature("Persistence.Saving")]
		static public void Serialize(string filename, object o, Type[] extraTypes)
		{

			// Writing the file requires a TextWriter.
			XmlTextWriter writer = new XmlTextWriter(filename, Encoding.UTF8);
			writer.Formatting = Formatting.Indented;

			Serialize(writer,o,extraTypes);

			writer.Close();
		}
		[Feature("Persistence.Saving")]
		static public void Serialize(XmlWriter writer, object o, Type[] extraTypes)
		{
			//check if the object implements ICustomSerialization
			if(o is ICustomSerialization)
			{
				((ICustomSerialization)o).Serialize(writer, extraTypes);
			}
			else
			{
				XmlSerializer s = new XmlSerializer(o.GetType(), extraTypes);

				// Serialize the class
				s.Serialize(writer, o);
			}
		}
		[Feature("Persistence.Loading")]
		static public object Deserialize(string filename, Type t)
		{
			return Deserialize(filename,t,new Type[]{});
		}
		[Feature("Persistence.Loading")]
		static public object Deserialize(string filename, Type t, Type[] extraTypes)
		{
			object ret = null;
			// Set up the reader
			XmlTextReader reader = new XmlTextReader(filename);
			ret = Deserialize(reader,t,extraTypes);
			reader.Close();

			return ret;
			
		}
		[Feature("Persistence.Loading")]
		static public object Deserialize(XmlReader reader, Type t)
		{
			return Deserialize(reader,t,new Type[]{});
		}
		[Feature("Persistence.Loading")]
		static public object Deserialize(XmlReader reader, Type t, Type[] extraTypes)
		{
			object ret=null;

			MethodInfo mi = t.GetMethod("Deserialize");
			if(mi!=null)
			{
				if(!mi.IsPublic || !mi.IsStatic)
					mi = null;
				ParameterInfo[] pi = mi.GetParameters();
				if(pi==null || pi.Length!=2 || pi[0].ParameterType!=typeof(XmlReader) || pi[1].ParameterType!=typeof(Type[]))
					mi = null;
			}
			
			//check if the object implements a custom Deserialize method
			if(mi!=null)
			{
				ret = mi.Invoke(null,new object[]{reader,extraTypes});
			}
			else
			{
				//Create the serializer
				XmlSerializer s = new XmlSerializer(t, extraTypes);

				// Deserialize the object.
				ret = s.Deserialize(reader);
			}
			//check if the object implements IInitAfterDeserialization
			if(ret is IInitAfterDeserialization)
				((IInitAfterDeserialization)ret).InitAfterDeserialization();

			return ret;
		}

	}	//end class XmlSerializerHelper
}//end namespace Columbia.Utilities
