///////////////////////////////////////////////////////////////////////
//  Copyright (C) 2005 Columbia University.  All Rights Reserved.
//
//      $Header: /Goblin/Engine/Devices/InputDeviceDriver.cs 6     1/12/07 2:23p Eaddy $
//
//    File name:	InputDeviceDriver.cs
//   Created by:	John Waugh
//   Created on:	04/08/2005
//
//      Purpose:	
//
//  Description:	
//
///////////////////////////////////////////////////////////////////////
/**
 * $Log: /Goblin/Engine/Devices/InputDeviceDriver.cs $
 * 
 * 6     1/12/07 2:23p Eaddy
 * o After annotating all statements with their corresponding requirements
 * 
 * 5     9/29/06 6:18p Eaddy
 * o Annotated program elements with the concern they realize
 * 
 * 4     5/31/05 7:12p Eaddy
 * o Fixed bug where tracker would fail but we thought it succeeded
 * 
 * 3     5/28/05 1:20p Eaddy
 * Changed copyright
 * 
 * 2     5/07/05 12:46p Eaddy
 * o Added comment header
 * o Now supports IDisposable
 */

using Phx.Aop.Attributes;

using System;
using System.Xml.Serialization;

namespace Goblin.Engine.Devices
{
	/// <summary>
	/// This is the base class for any provider of InputDevices
	/// Your driver inherits from this class, and provides one or more InputDevices via the addMappings function
	/// </summary>
	
	//[Serializable]
	[Feature("Input")]
	[Requirement("Device.Device abstraction (#100)")]
	[Requirement("Device.Dynamic device discovery (#105)")]
	[Requirement("Configuration.Dynamic reconfiguration (#405)")]
	abstract public class InputDeviceDriver : IDisposable
	{
		public InputDeviceDriver()
		{
			//[Feature("Persistence")]
			identifier = "<InputDeviceDriver_uninitialized_identifier>";
		}

		[XmlElement("driver_uid")]
		[Feature("Persistence")]
		[Requirement("Configuration.Device configuration files (#135)")]
		public string identifier;

		/// <summary>
		/// This function returns the InputDevice corresponding to a name
		/// </summary>
		/// <param name="input_uid"></param>
		/// <returns></returns>
		[Feature("Persistence")]
		abstract public InputDevice GetInputByName(string input_uid);

		/// <summary>
		/// Subclases should use this function to update all mapped inputs.
		/// </summary>
		/// <param name="elapsed_seconds"></param>
		abstract public bool Update(float elapsed_seconds);

		/// <summary>
		/// This function is available in case your class needs to do additional work to completely
		/// restore its state after being deserialized
		/// </summary>
		[Feature("Persistence.Loading")]
		[Requirement("Configuration.Device configuration files (#135)")]
		virtual public void InitAfterDeserialization()
		{}

		[Requirement("Nonfunctional.Clean shutdown (#1405)")]
		virtual public void Dispose()
		{ }
	}
}
