///////////////////////////////////////////////////////////////////////
//  Copyright (C) 2005 Columbia University.  All Rights Reserved.
//
//      $Header: /Goblin/Engine/Devices/InputDevice_6DOF.cs 7     2/02/07 5:18a Eaddy $
//
//    File name:	InputDevice_6DOF.cs
//   Created by:	John Waugh
//   Created on:	04/08/2005
//
//      Purpose:	
//
//  Description:	
//
///////////////////////////////////////////////////////////////////////
/**
 * $Log: /Goblin/Engine/Devices/InputDevice_6DOF.cs $
 * 
 * 7     2/02/07 5:18a Eaddy
 * o Final assignment used for ASOC submission
 * 
 * 6     1/12/07 2:23p Eaddy
 * o After annotating all statements with their corresponding requirements
 * 
 * 5     9/29/06 6:18p Eaddy
 * o Annotated program elements with the concern they realize
 * 
 * 4     5/28/05 1:20p Eaddy
 * Changed copyright
 * 
 * 3     5/25/05 6:28p Eaddy
 * o Removed unused class
 * 
 * 2     5/07/05 12:45p Eaddy
 * o Added comment header
 */

using Phx.Aop.Attributes;

using System;
using Microsoft.DirectX;

namespace Goblin.Engine.Devices
{
	[Feature("SceneGraph.Movement")]
	[Requirement("Device.Device abstraction (#100)")]
	[Requirement("Device.Position and orientation trackers (#150)")]
    [Requirement("Rendering.Coordinate system independence (#715)")]
    [Requirement("Rendering.Multiple coordinate systems (#705)")]
    [Requirement("Rendering.Relative coordinate systems (#710)")]
	public delegate Matrix MatrixDelegate();

	[Feature("SceneGraph.Movement")]
	[Requirement("Device.Device abstraction (#100)")]
	[Requirement("Device.Position and orientation trackers (#150)")]
    [Requirement("Rendering.Coordinate system independence (#715)")]
    [Requirement("Rendering.Multiple coordinate systems (#705)")]
    [Requirement("Rendering.Relative coordinate systems (#710)")]
    public delegate Matrix TransformDelegate(Matrix mat);

	/// <summary>
	/// Subclass of InputDevice, providing 6 degrees of freedom information about some data source
	/// </summary>
	[Feature("Input")]
	[Feature("SceneGraph.Movement")]
	[Requirement("Device.Device abstraction (#100)")]
	[Requirement("Device.Position and orientation trackers (#150)")]
    [Requirement("Rendering.Coordinate system independence (#715)")]
    [Requirement("Rendering.Multiple coordinate systems (#705)")]
    [Requirement("Rendering.Relative coordinate systems (#710)")]
	public class InputDevice_6DOF : InputDevice
	{
		public InputDevice_6DOF()
		{}
		
		/*
		 * The following functions should be overridden by subclasses.
		 * Default behavior is no transformation (identity matrix)
		 * */
		
		virtual public Matrix GetTransform()
		{ return Matrix.Identity; }

		#region "TODO ?"
		
		// TODO ? : add these 6 functions in the future
		// perhaps have quaternion formatted functions as well.
		//public float XTrans()
		//{ return 0; }
		//public float YTrans()
		//{ return 0; }
		//public float ZTrans()
		//{ return 0; }
		//public float XRot()
		//{ return 0; }
		//public float YRot()
		//{ return 0; }
		//public float ZRot()
		//{ return 0; }
		
		#endregion
		
		// generally, subclasses won't have to touch the functions below.
		// They can just override GetTransform and these will be correct.
		//

		virtual public MatrixDelegate Transform()
		{ return new MatrixDelegate(GetTransform); }
		
		virtual	public TransformDelegate TransformBefore()
		{ return new TransformDelegate(GetTransformedBefore); }
		
		virtual public TransformDelegate TransformAfter()
		{ return new TransformDelegate(GetTransformedAfter); }

		virtual public Matrix GetTransformedBefore(Matrix inMat)
		{ return GetTransform() * inMat; }
		virtual public Matrix GetTransformedAfter(Matrix inMat)
		{ return inMat * GetTransform(); }

	}
}
