///////////////////////////////////////////////////////////////////////
//  Copyright (C) 2005 Columbia University.  All Rights Reserved.
//
//      $Header: /Goblin/Engine/Devices/InterSense/InterSense.cs 35    2/02/07 5:18a Eaddy $
//
//    File name:	InterSense.cs
//   Created by:	Marc Eaddy
//   Created on:	02/04/2005
//
//      Purpose:
//
//  Description:
//
///////////////////////////////////////////////////////////////////////
/**
 * $Log: /Goblin/Engine/Devices/InterSense/InterSense.cs $
 * 
 * 35    2/02/07 5:18a Eaddy
 * o Final assignment used for ASOC submission
 * 
 * 34    1/22/07 12:13a Eaddy
 * o Cleaned up some redundant and missing annotations
 * 
 * 33    1/19/07 5:32p Eaddy
 * o Added Requirement annotations
 * 
 * 32    1/12/07 2:23p Eaddy
 * o After annotating all statements with their corresponding requirements
 * 
 * 31    10/10/06 3:15p Eaddy
 * o Removed redundant concern annotations
 * o We now have 100% concern annotation coverage
 * 
 * 30    9/29/06 6:18p Eaddy
 * o Annotated program elements with the concern they realize
 * 
 * 29    5/02/06 11:03a Eaddy
 * o Moved MyTrace into separate library
 * 
 * 28    5/31/05 7:12p Eaddy
 * o Fixed bug where tracker would fail but we thought it succeeded
 * 
 * 27    5/30/05 9:48p Eaddy
 * o Trivial
 * 
 * 26    5/28/05 1:21p Eaddy
 * Changed copyright
 * 
 * 25    5/07/05 12:54p Eaddy
 * o No longer crashes if isense.dll is not found
 * o Added Dispose() to free up socket and serial port
 * 
 * 24    4/15/05 9:56p Eaddy
 * o Should default to checking for directly connected tracker if network
 * connection fails
 * 
 * 23    4/08/05 1:44p Jrw2005
 * 
 * 22    2/26/05 12:15a Eaddy
 * o Changed log message
 * 
 * 21    2/26/05 12:14a Eaddy
 * o Now checks if the socket is connected before continuing to request
 * data
 * 
 * 20    2/25/05 5:09p Eaddy
 * o Connect to braque by default
 * 
 * 19    2/25/05 4:02p Jrw2005
 * 
 * 18    2/25/05 1:49p Jrw2005
 * added InterSenseSocket stuff
 * 
 * 17    2/22/05 11:59p Eaddy
 * o Prepped for adding InterSenseSocket class
 * 
 * 16    2/22/05 11:30p Eaddy
 * o Now automatically initializes
 * o Fixed bug where we were getting quaternions from ISense. We now
 * specifically request Euler.
 * 
 * 15    2/18/05 9:59p Eaddy
 * o Created a generic 6DOF controller class that supports 6DOF data from
 * a DX camera or an IS900 tracker
 * 
 * 14    2/18/05 8:23p Eaddy
 * o Now checks that we are inited before retrieving the data
 * 
 * 13    2/18/05 8:21p Eaddy
 * o Converted functions to use 'out' to avoid an unecessary marshalling
 * pass
 * o Converted handles to use IntPtr instead of int (more portable)
 * o Updated comments
 * 
 * 12    2/18/05 3:55p Eaddy
 * o Got rid of extra float array copies
 * 
 * 11    2/18/05 11:54a Edp2002
 * Array Fixes for Station
 * 
 * 10    2/18/05 1:08p Jrw2005
 * head transform work?
 * 
 * 9     2/18/05 11:34a Eaddy
 * o Fixed bug where rotation was wrong when you rolled your head while
 * looking at the door
 * 
 * 8     2/17/05 3:35p Eaddy
 * o Now tracked camera works
 * 
 * 7     2/11/05 9:47p Eaddy
 * o Fixed isense transformation. Now tracked objects appear in the right
 * place and orientation.
 * 
 * 6     2/11/05 9:44p Eaddy
 * o Right before fixing the isense matrix
 * 
 * 5     2/11/05 8:33p Eaddy
 * o Doesn't work
 * 
 * 4     2/04/05 7:34p Eaddy
 * o Still doesn't work
 * 
 * 3     2/04/05 5:49p Eaddy
 * o Trying different things to get the transformation to look right
 * 
 * 2     2/04/05 3:10p Eaddy
 * o Now reads tracker data correctly and transforms it  Doesn't quite
 * work yet.
 * 
 * 1     2/04/05 2:50p Eaddy
 * o Added InterSense class
 */

using Phx.Aop.Attributes;

using System;
using System.Diagnostics;
using System.Collections;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

using System.Xml.Serialization;
using Columbia.Utilities;

namespace Goblin.Engine.Devices
{
	/// <summary>
	/// Summary description for InterSense.
	/// </summary>
	[Feature("Input")]
	[Requirement("Device.Position and orientation trackers (#150)")]
	[Requirement("Device.Local device state (#125)")]
	[Requirement("Device.Device abstraction (#100)")]
	[Requirement("Device.Dynamic device discovery (#105)")]
	[Requirement("Device.Head mounted displays (#195)")]
	[Requirement("3D UI.Optical see-through HMD support (#910)")]
	[Requirement("Device.Networked input devices and trackers (#290)")]
	[Requirement("Device.Network-level device integration (#240)")]
	[Requirement("Rendering.Coordinate system independence (#715)")]
	public class InterSense : InputDeviceDriver, IDisposable
	{
		[Requirement("Device.Position and orientation trackers (#150)")]
		[Requirement("Device.Local device state (#125)")]
		[Requirement("Device.Networked input devices and trackers (#290)")]
		[Requirement("Device.Network-level device integration (#240)")]
		public const int nINDEX_STATION_1					= 0;
		
		[Requirement("Device.Position and orientation trackers (#150)")]
		[Requirement("Device.Local device state (#125)")]
		[Requirement("Device.Networked input devices and trackers (#290)")]
		[Requirement("Device.Network-level device integration (#240)")]
		public const int nINDEX_STATION_2_HAND				= 1; // Hand tracker

		[Requirement("Device.Position and orientation trackers (#150)")]
		[Requirement("Device.Local device state (#125)")]
		[Requirement("Device.Networked input devices and trackers (#290)")]
		[Requirement("Device.Network-level device integration (#240)")]
		[Requirement("Device.Head mounted displays (#195)")]
		[Requirement("3D UI.Optical see-through HMD support (#910)")]
		public const int nINDEX_STATION_3_HMD				= 2; // Tracker attached to HMD

		[Requirement("Device.Position and orientation trackers (#150)")]
		[Requirement("Device.Local device state (#125)")]
		[Requirement("Device.Networked input devices and trackers (#290)")]
		[Requirement("Device.Network-level device integration (#240)")]
		public const int nINDEX_STATION_4_WIRELESS			= 3; // Wireless

		[XmlElement("connection")]
		[Feature("Persistence")]
		[Feature("Input.Network")]
		[Requirement("Device.Position and orientation trackers (#150)")]
		[Requirement("Device.Networked input devices and trackers (#290)")]
		[Requirement("Device.Network-level device integration (#240)")]
		[Requirement("Configuration.Device configuration files (#135)")]
		public ConnectionInfo connInfo;
		
		[XmlArray("stations")]
		[XmlArrayItem(typeof(StationInfo))]
		
		[Feature("Persistence")]
		[Requirement("Device.Position and orientation trackers (#150)")]
		[Requirement("Device.Local device state (#125)")]
		[Requirement("Device.Head mounted displays (#195)")]
		[Requirement("3D UI.Optical see-through HMD support (#910)")]
		[Requirement("Device.Networked input devices and trackers (#290)")]
		[Requirement("Device.Network-level device integration (#240)")]
		[Requirement("Configuration.Device configuration files (#135)")]
		[Requirement("Rendering.Coordinate system independence (#715)")]
		// Must be "public" for XML serialization to work?
		public StationInfo[]	stationInfo;

		[Requirement("Device.Dynamic device discovery (#105)")]
		[Requirement("Device.Device abstraction (#100)")]
		private Hashtable		stationHash = new Hashtable(ISDllBridge.ISD_MAX_STATIONS);

		[Requirement("Device.Position and orientation trackers (#150)")]
		[Requirement("Device.Local device state (#125)")]
		[Requirement("Device.Head mounted displays (#195)")]
		[Requirement("3D UI.Optical see-through HMD support (#910)")]
		[Requirement("Device.Networked input devices and trackers (#290)")]
		[Requirement("Device.Network-level device integration (#240)")]
		private StationArray	stationArray;

		//depending on our connection type, we will use
		[Requirement("Device.Position and orientation trackers (#150)")]
		[Requirement("Device.Head mounted displays (#195)")]
		[Requirement("3D UI.Optical see-through HMD support (#910)")]
		private IntPtr isenseHandle = IntPtr.Zero;
		[Feature("Input.Network")]
		[Requirement("Device.Position and orientation trackers (#150)")]
		[Requirement("Device.Head mounted displays (#195)")]
		[Requirement("3D UI.Optical see-through HMD support (#910)")]
		[Requirement("Device.Networked input devices and trackers (#290)")]
		[Requirement("Device.Network-level device integration (#240)")]
		private InterSenseSocket isenseSocket = null;
		
		#region "StationArray internal class"
		/// <summary>
		/// Manages the array of stations. Uses lazy allocation to only allocate
		/// and update stations that are actually used.
		/// </summary>
		[Requirement("Device.Position and orientation trackers (#150)")]
		[Requirement("Device.Local device state (#125)")]
		[Requirement("Device.Head mounted displays (#195)")]
		[Requirement("3D UI.Optical see-through HMD support (#910)")]
		[Requirement("Device.Networked input devices and trackers (#290)")]
		[Requirement("Device.Network-level device integration (#240)")]
		[Requirement("Configuration.Device configuration files (#135)")]
		public class StationArray
		{
			public StationArray(StationInfo[] info)
			{
				this.info = info;
			}

			public bool isActive(long index)
			{
				return stations[index] != null;
			}

			public void SetData(ISDllBridge.ISD_TRACKER_DATA_TYPE dataISense)
			{
				foreach (InterSenseStation station in stations)
				{
					if (station != null)
						station.SetData(dataISense);
				}
			}

			// Array indexer
			public InterSenseStation this[long index]
			{
				get
				{
					//[Requirement("Operational.Minimum frame rate of 15 fps (#1305)")]
					//[Requirement("Operational.Maximum user input latency of 1 second (#1310)")]
					//[Requirement("Operational.Target frame rate of 60 fps (#1300)")]
					//[Requirement("Device.Position and orientation trackers (#150)")]
					//[Requirement("Device.Local device state (#125)")]
					//[Requirement("Device.Head mounted displays (#195)")]
					//[Requirement("3D UI.Optical see-through HMD support (#910)")]
					//[Requirement("Device.Networked input devices and trackers (#290)")]
					//[Requirement("Device.Network-level device integration (#240)")]
					//[Requirement("Configuration.Device configuration files (#135)")]
					if (stations[index] == null)
						stations[index] = new InterSenseStation(index,info[index].transform);

					return stations[index];
				}
			}

			// DATA
			private StationInfo[]	info = null;
			private InterSenseStation[] stations = new InterSenseStation[ISDllBridge.ISD_MAX_STATIONS];
		}
		#endregion

		#region "StationInfo internal class"
		/// <summary>
		/// This class maintains a mapping of name->index, for use in serialization
		/// </summary>
		[Requirement("Device.Position and orientation trackers (#150)")]
		[Requirement("Device.Head mounted displays (#195)")]
		[Requirement("3D UI.Optical see-through HMD support (#910)")]
		[Requirement("Device.Networked input devices and trackers (#290)")]
		[Requirement("Device.Network-level device integration (#240)")]
		[Requirement("Configuration.Device configuration files (#135)")]
		public class StationInfo
		{
			public StationInfo()
			{}
			public StationInfo(string name, int index, Matrix headTransform)
			{
				this.name = name;
				this.index = index;
				this.transform = headTransform;
			}
			[Feature("Persistence")]
			[XmlElement]
			public string	name;
			[Feature("Persistence")]
			[XmlElement]
			public long		index;
			[Feature("Persistence")]
			[XmlElement("head_transform")]
			public Matrix	transform;
		}
		#endregion
		
		#region "ConnectionInfo internal class"
		[Requirement("Device.Networked input devices and trackers (#290)")]
		[Requirement("Device.Network-level device integration (#240)")]
		[Requirement("Configuration.Device configuration files (#135)")]
		[Requirement("Rendering.Coordinate system independence (#715)")]
		public class ConnectionInfo
		{
			public ConnectionInfo()
			{}
			public ConnectionInfo(string host, int port, bool preferNetwork, bool tryBoth)
			{
				//[Feature("Input.Network")]
				this.preferNetwork = preferNetwork;
				//[Feature("Input.Network")]
				this.tryBoth = tryBoth;
				this.host = host;
				this.port = port;
			}
			
			[Feature("Persistence")]
			[XmlElement]
			public string	host;

			[Feature("Persistence")]
			[XmlElement]
			public int		port;

			[Feature("Persistence")]
			[XmlElement("prefer_network")]
			[Feature("Input.Network")]
			public bool		preferNetwork;

			[Feature("Persistence")]
			[XmlElement("try_both")]
			[Feature("Input.Network")]
			public bool		tryBoth;
		}
		#endregion

		public InterSense()
		{ }

		public InterSense(	[Requirement("Device.Dynamic device discovery (#105)")] 
		                  	string unique_identifier)
		{
			//[Requirement("Device.Dynamic device discovery (#105)")]
			identifier = unique_identifier;
			//[Feature("Input.Network")]

			//[Requirement("Device.Networked input devices and trackers (#290)")]
			//[Requirement("Device.Network-level device integration (#240)")]
			connInfo = new ConnectionInfo("braque.cs.columbia.edu",4000,true,true);

			//[Requirement("Device.Position and orientation trackers (#150)")]
			//[Requirement("Device.Local device state (#125)")]
			//[Requirement("Device.Device abstraction (#100)")]
			//[Requirement("Device.Dynamic device discovery (#105)")]
			//[Requirement("Device.Head mounted displays (#195)")]
			//[Requirement("3D UI.Optical see-through HMD support (#910)")]
			//[Requirement("Device.Networked input devices and trackers (#290)")]
			//[Requirement("Device.Network-level device integration (#240)")]
			{
				stationInfo = new StationInfo[ISDllBridge.ISD_MAX_STATIONS];
				for(int index=0;index<stationInfo.Length;index++)
				{
					string name = "station_" + index;
					stationInfo[index] = new StationInfo(name, index, Matrix.Translation(0, -0.1524f, -0.1397f));
					stationHash[name] = index;
				}
				stationArray = new StationArray(stationInfo);
			}
		}


		[Requirement("Nonfunctional.Clean shutdown (#1405)")]
		public override void Dispose()
		{
			// Close the socket (if open)
			//[Feature("Input.Network")]
			//[Requirement("Device.Position and orientation trackers (#150)")]
			//[Requirement("Device.Head mounted displays (#195)")]
			//[Requirement("3D UI.Optical see-through HMD support (#910)")]
			//[Requirement("Device.Networked input devices and trackers (#290)")]
			//[Requirement("Device.Network-level device integration (#240)")]
			//[Requirement("Nonfunctional.Clean shutdown (#1405)")]
			if (isenseSocket != null)
			{
				isenseSocket.Dispose();
				isenseSocket = null;
			}

			// Close the serial port (if open)
			//[Requirement("Device.Position and orientation trackers (#150)")]
			//[Requirement("Device.Local device state (#125)")]
			//[Requirement("Device.Head mounted displays (#195)")]
			//[Requirement("3D UI.Optical see-through HMD support (#910)")]
			//[Requirement("Nonfunctional.Clean shutdown (#1405)")]
			if (isenseHandle != IntPtr.Zero)
			{
				ISDllBridge.ISD_CloseTracker(isenseHandle);
				isenseHandle = IntPtr.Zero;
			}
		}

		/// <summary>
		/// This gets called after deserialization
		/// </summary>
		/// <param name="idm"></param>
		[Feature("Persistence.Loading")]
		override public void InitAfterDeserialization()
		{
			//build the hash from the name mapping
			foreach(StationInfo i in stationInfo)
				stationHash[i.name] = i.index;
			stationArray = new StationArray(stationInfo);
		}

		/// <summary>
		/// Given a station name, returns the InputDevice that accesses its data
		/// </summary>
		/// <param name="input_uid"></param>
		/// <returns></returns>
		override public InputDevice GetInputByName(string input_uid)
		{
			//do lazy initialization
			if (!Init())
				//[Feature("EC")]
				return null;
			
			//[Feature("EC")]
			if (stationHash[input_uid] == null)
				return null;

			//because of lazy allocation, we need to access the stationArray once first
			//to make it created, then call Update to fill in data for it
			//then we can return the station
			InterSenseStation ret = stationArray[(long)stationHash[input_uid]];
			//[Feature("Logging")]
			MyTrace.Log("Performing initial retrieval of data.");
			if (!Update(0))
				//[Feature("EC")]
				return null;
			
			return ret;
		}

		#region "eliminate later"

		// Force access to the station array to go through
		// this Property so that we can automatically initialize
		// InterSense the first time.
		//public StationArray Stations
		//{
		//	get
		//	{
		//		Init();
		//		return stationArray;
		//	}
		//}

		#endregion

		public bool Init()
		{
			if (IsInited())
			{
				return true;
			}

			//[Feature("Logging")]
			MyTrace.Log(TraceLevel.Info, "Connecting to InterSense...");

			// First try to connect to the InterSense network server
			//[Feature("Input.Network")]
			bool bRet = InitSocket();
			//[Feature("Input.Network")]
			if (!bRet)
			{
				// If that fails, try connecting directly using the serial port driver
				bRet = InitDriver();
			}

			//[Feature("Logging")]
			if (bRet)
			{
				MyTrace.Log(TraceLevel.Info, "InterSense connection successful.");

			}
			else
			{
				MyTrace.Log(TraceLevel.Error,
				            "Unable to connect to InterSense head tracker using socket or serial port driver.");
			}

			return true;
		}

		[Feature("Input.Network")]
		[Requirement("Device.Position and orientation trackers (#150)")]
		[Requirement("Device.Head mounted displays (#195)")]
		[Requirement("3D UI.Optical see-through HMD support (#910)")]
		[Requirement("Device.Networked input devices and trackers (#290)")]
		[Requirement("Device.Network-level device integration (#240)")]
		private bool InitSocket()
		{
			if(isenseSocket == null)
				isenseSocket = new InterSenseSocket();

			//[Feature("EC")]
            //[Requirement("Debugging.Exception/error detection (#625)")]
            //[Requirement("Debugging.Exception/error handling (#630)")]
			Debug.Assert(!isenseSocket.IsConnected());

			return isenseSocket.Connect(connInfo.host,connInfo.port);
		}

		[Requirement("Device.Position and orientation trackers (#150)")]
		[Requirement("Device.Local device state (#125)")]
		[Requirement("Device.Head mounted displays (#195)")]
		[Requirement("3D UI.Optical see-through HMD support (#910)")]
		private bool InitDriver()
		{
			// Step 1:	Connect to InterSense using the serial port driver

			//[Feature("EC")]
            //[Requirement("Debugging.Exception/error detection (#625)")]
            //[Requirement("Debugging.Exception/error handling (#630)")]
			Debug.Assert(isenseHandle == IntPtr.Zero);

			//[Feature("EC")]
			//[Requirement("Debugging.Exception/error detection (#625)")]
			try
			{
				//[Requirement("Device.Position and orientation trackers (#150)")]
				//[Requirement("Device.Local device state (#125)")]
				//[Requirement("Device.Head mounted displays (#195)")]
				//[Requirement("3D UI.Optical see-through HMD support (#910)")]
				isenseHandle = ISDllBridge.ISD_OpenTracker(IntPtr.Zero, 0, true, false);
			}
            //:block[Feature("EH")]
			//:block[Requirement("Debugging.Exception/error handling (#630)")]
			catch (DllNotFoundException ex)
			{
				//[Feature("Logging")]
				//[Requirement("Debugging.Logging (#600)")]
				//[Requirement("Debugging.Exception/error handling (#630)")]
				MyTrace.Log(TraceLevel.Warning,
				            "Failed to connect to InterSense via serial port: {0}",
				            ex.Message);
			}

			//[Feature("EC")]
			//[Requirement("Debugging.Exception/error detection (#625)")]
			if (isenseHandle == IntPtr.Zero)
			{
				//[Requirement("Debugging.Exception/error handling (#630)")]
				return false;
			}

			// Step 2:	Tell InterSense to give us euler

			for(short i = 1; i <= ISDllBridge.ISD_MAX_STATIONS; ++i)
			{
				ISDllBridge.ISD_STATION_INFO_TYPE stationInfoType;

				bool bRet = ISDllBridge.ISD_GetStationConfig(	isenseHandle,
				                                             out stationInfoType,
				                                             i,
				                                             true);
				//[Feature("EC")]
				//[Requirement("Debugging.Exception/error detection (#625)")]
				if (!bRet)
				{
					//[Feature("Logging")]
					//[Requirement("Debugging.Exception/error handling (#630)")]
					//[Requirement("Debugging.Logging (#600)")]
					MyTrace.Log(TraceLevel.Error, "ISD_GetStationConfig({0}) failed.",
					            i);
					//[Requirement("Debugging.Exception/error handling (#630)")]
					return false;
				}

				stationInfoType.AngleFormat = ISDllBridge.ISD_EULER;

				bRet = ISDllBridge.ISD_SetStationConfig(		isenseHandle,
				                                        ref stationInfoType,
				                                        i,
				                                        true);
				//[Feature("EC")]
				//[Requirement("Debugging.Exception/error detection (#625)")]
				if (!bRet)
				{
					//[Feature("Logging")]
					//[Requirement("Debugging.Exception/error handling (#630)")]
					//[Requirement("Debugging.Logging (#600)")]
					MyTrace.Log(TraceLevel.Error, "ISD_SetStationConfig({0}) failed.",
					            i);
					//[Requirement("Debugging.Exception/error handling (#630)")]
					return false;
				}
			}

			//[Requirement("Debugging.Exception/error handling (#630)")]
			return true;
		}

		public bool IsInited()
		{
			return isenseSocket != null || isenseHandle != IntPtr.Zero;
		}

		/// <summary>
		/// Updates all its mapped inputs
		/// </summary>
		/// <param name="elapsedTime"></param>
		[Requirement("Device.Position and orientation trackers (#150)")]
		[Requirement("Device.Local device state (#125)")]
		[Requirement("Device.Head mounted displays (#195)")]
		[Requirement("3D UI.Optical see-through HMD support (#910)")]
		[Requirement("Device.Networked input devices and trackers (#290)")]
		[Requirement("Device.Network-level device integration (#240)")]
		override public bool Update(float elapsed_seconds)
		{
			//[Feature("EC")]
			//[Requirement("Debugging.Exception/error detection (#625)")]
			if (!IsInited())
			{
				//[Requirement("Debugging.Exception/error handling (#630)")]
				return false;
			}

			ISDllBridge.ISD_TRACKER_DATA_TYPE dataISense;
			bool bSuccess = false;

			// Try the socket

			//[Feature("Input.Network")]
			//[Requirement("Device.Position and orientation trackers (#150)")]
			//[Requirement("Device.Head mounted displays (#195)")]
			//[Requirement("3D UI.Optical see-through HMD support (#910)")]
			//[Requirement("Device.Networked input devices and trackers (#290)")]
			//[Requirement("Device.Network-level device integration (#240)")]
			if (isenseSocket != null)
			{
				//[Feature("EC")]
				//[Requirement("Debugging.Exception/error detection (#625)")]
				if (!isenseSocket.IsConnected())
				{
					//[Requirement("Debugging.Exception/error handling (#630)")]
					return false;
				}

				// Pass the station array so that we know which stations to
				// request data for in the network message
				//[Requirement("Device.Local device state (#125)")]
				bSuccess = isenseSocket.GetData(stationArray, out dataISense);
				//[Requirement("Debugging.Exception/error detection (#625)")]
				if (bSuccess)
				{
					//[Requirement("Device.Position and orientation trackers (#150)")]
					//[Requirement("Device.Local device state (#125)")]
					//[Requirement("Device.Head mounted displays (#195)")]
					//[Requirement("3D UI.Optical see-through HMD support (#910)")]
					//[Requirement("Device.Networked input devices and trackers (#290)")]
					//[Requirement("Device.Network-level device integration (#240)")]
					stationArray.SetData(dataISense);
					//[Requirement("Debugging.Exception/error handling (#630)")]
					return true;
				}
				//:block[Requirement("Debugging.Exception/error handling (#630)")]
				else
				{
					// If that fails, try connecting directly using the serial port driver
					InitDriver();
					isenseSocket = null;
				}
			}

			// Try getting the data via serial port

			//[Requirement("Debugging.Exception/error detection (#625)")]
			if (!bSuccess && isenseHandle != IntPtr.Zero)
			{
				//[Requirement("Device.Position and orientation trackers (#150)")]
				//[Requirement("Device.Head mounted displays (#195)")]
				//[Requirement("3D UI.Optical see-through HMD support (#910)")]
				//[Requirement("Device.Local device state (#125)")]
				bSuccess = ISDllBridge.ISD_GetData(isenseHandle, out dataISense);
				//[Requirement("Debugging.Exception/error detection (#625)")]
				if (bSuccess)
				{
					//[Requirement("Device.Position and orientation trackers (#150)")]
					//[Requirement("Device.Local device state (#125)")]
					//[Requirement("Device.Head mounted displays (#195)")]
					//[Requirement("3D UI.Optical see-through HMD support (#910)")]
					stationArray.SetData(dataISense);
				}
			}

			//[Requirement("Debugging.Exception/error handling (#630)")]
			return bSuccess;
		}
	}
}
