///////////////////////////////////////////////////////////////////////
//  Copyright (C) 2005 Columbia University.  All Rights Reserved.
//
//      $Header: /Goblin/Engine/Devices/InterSense/InterSenseSocket.cs 16    2/02/07 5:18a Eaddy $
//
//    File name:	InterSenseSocket.cs
//   Created by:	John Waugh
//   Created on:	02/25/2005
//
//      Purpose:	
//
//  Description:	
//
///////////////////////////////////////////////////////////////////////
/**
 * $Log: /Goblin/Engine/Devices/InterSense/InterSenseSocket.cs $
 * 
 * 16    2/02/07 5:18a Eaddy
 * o Final assignment used for ASOC submission
 * 
 * 15    1/22/07 12:13a Eaddy
 * o Cleaned up some redundant and missing annotations
 * 
 * 14    1/19/07 5:32p Eaddy
 * o Added Requirement annotations
 * 
 * 13    1/12/07 2:23p Eaddy
 * o After annotating all statements with their corresponding requirements
 * 
 * 12    10/10/06 3:15p Eaddy
 * o Removed redundant concern annotations
 * o We now have 100% concern annotation coverage
 * 
 * 11    9/29/06 6:18p Eaddy
 * o Annotated program elements with the concern they realize
 * 
 * 10    5/02/06 11:03a Eaddy
 * o Moved MyTrace into separate library
 * 
 * 9     5/28/05 1:21p Eaddy
 * Changed copyright
 * 
 * 8     5/08/05 11:10p Eaddy
 * o Fixed bug where we would block waiting for a message that never came
 * 
 * 7     5/07/05 12:48p Eaddy
 * o Minor changes
 * 
 * 6     4/15/05 5:12p Eaddy
 * o Fixed bug where UDP requests for multiple stations was not parsed
 * properly
 * causing an infinite loop
 * 
 * 5     4/08/05 1:44p Jrw2005
 * 
 * 4     2/26/05 12:14a Eaddy
 * o Fixed problem where a failed connection caused app to exit with an
 * exception
 * o Added comment header
 * o Now uses MyTrace.Log so messages are logged
 */

using Phx.Aop.Attributes;

using System;
using System.Text;
using System.IO;
using System.Net;
using System.Net.Sockets;
using System.Diagnostics;
using System.Text.RegularExpressions;
using System.Runtime.InteropServices;

using Columbia.Utilities;

namespace Goblin.Engine.Devices
{
	/// <summary>
	/// Helper class for accessing the Poll() method in UpdClient
	/// </summary>
	[Feature("Network")]
	[Requirement("Device.Network-level device integration (#240)")]
	[Requirement("Device.Networked input devices and trackers (#290)")]
	public class MyUdpClient : UdpClient
	{
		public MyUdpClient(string hostname, int port)
			: base(hostname, port)
		{ }

		public bool PollForData(int secs)
		{
			return Client.Poll(secs * 1000000, SelectMode.SelectRead);
		}
	}

	/// <summary>
	/// Summary description for InterSenseSocket.
	/// </summary>
	[Feature("Input.Network")]
	[Requirement("Device.Network-level device integration (#240)")]
	[Requirement("Device.Networked input devices and trackers (#290)")]
	public class InterSenseSocket : IDisposable
	{
		public InterSenseSocket()
		{
			udp = null;
			sourcePoint = null;
		}

		[Requirement("Device.Network-level device integration (#240)")]
		[Requirement("Device.Networked input devices and trackers (#290)")]
		[Requirement("Nonfunctional.Clean shutdown (#1405)")]
		public void Dispose()
		{
			Close();
		}

		//Connect to the data source on the given socket
		public bool Connect(string server_hostname, int server_port)
		{
			hostname = server_hostname;

			//[Feature("Logging")]
			//[Requirement("Debugging.Logging (#600)")]
			MyTrace.Log(TraceLevel.Info, "Connecting to {0}:{1}",
				server_hostname, server_port);

			//[Feature("EC")]
            //[Requirement("Debugging.Exception/error detection (#625)")]
			try
			{
				udp = new MyUdpClient(server_hostname, server_port);

				IPHostEntry host = Dns.GetHostByName(server_hostname);
				//[Feature("EC")]
				//[Requirement("Debugging.Exception/error detection (#625)")]
				if(host.AddressList.Length <= 0)
				{
					//[Feature("Logging")]
					//[Requirement("Debugging.Exception/error handling (#630)")]
					//[Requirement("Debugging.Logging (#600)")]
					MyTrace.Log(TraceLevel.Warning,
						"Could not find IP address of server '{0}'.",
						server_hostname);
					//[Requirement("Debugging.Exception/error handling (#630)")]
					//[Requirement("Nonfunctional.Clean shutdown (#1405)")]
					Close();
					//[Requirement("Debugging.Exception/error handling (#630)")]
					return false;
				}

				sourcePoint = new System.Net.IPEndPoint(host.AddressList[0],server_port);
				//[Feature("Logging")]
				//[Requirement("Debugging.Logging (#600)")]
				MyTrace.Log(TraceLevel.Info, "SourcePoint: "+sourcePoint.ToString());

				// NOTE: We should send a test message here so that we know right away if
				// ISenseServer is running (reduces delay).

				//[Requirement("Debugging.Exception/error handling (#630)")]
				return true;
			}
            //:block[Feature("EH")]
			//:block[Requirement("Debugging.Exception/error handling (#630)")]
			catch(SocketException ex)
			{
				// The help docs say that ex.ErrorCode will be the socket error code
				// but its wrong. The value should be WSAECONNRESET but instead its E_FAIL.
				//[Feature("Logging")]
				if (ex.Message.IndexOf("forcibly") > -1)
				{
					// ISenseServer died (CTRL^C)
					//[Requirement("Debugging.Exception/error handling (#630)")]
					//[Requirement("Debugging.Logging (#600)")]
					MyTrace.Log(TraceLevel.Warning,
						"InterSenseSocket::Connect() - ISenseServer '{0}' appears to have died.",
						server_hostname);
				}
				else
				{
					//[Requirement("Debugging.Logging (#600)")]
					//[Requirement("Debugging.Exception/error handling (#630)")]
					MyTrace.Log(TraceLevel.Info, "InterSenseSocket::Connect() - Error creating udp Connection to {0}:{1} : {2}",
						server_hostname,
						server_port,
						ex.ToString());
				}

				//[Requirement("Nonfunctional.Clean shutdown (#1405)")]
				//[Requirement("Debugging.Exception/error handling (#630)")]
				Close();
				//[Requirement("Debugging.Exception/error handling (#630)")]
				return false;
			}
            //:block[Feature("EH")]
            //:block[Requirement("Debugging.Exception/error handling (#630)")]
			catch(Exception ex)
			{
				//[Feature("Logging")]
	            //[Requirement("Debugging.Logging (#600)")]
				//[Requirement("Debugging.Exception/error handling (#630)")]
				MyTrace.Log(TraceLevel.Info, "InterSenseSocket::Connect() - Error creating udp Connection to {0}:{1} : {2}",
					server_hostname,
					server_port,
					ex.ToString());
				//[Requirement("Nonfunctional.Clean shutdown (#1405)")]
				Close();
				//[Requirement("Debugging.Exception/error handling (#630)")]
				return false;
			}
		}

		public bool IsConnected()
		{
			return udp != null;
		}

		[Requirement("Nonfunctional.Clean shutdown (#1405)")]
		public void Close()
		{
			if(udp!=null)
			{
				udp.Close();
				udp = null;
			}
		}

		public bool GetData(InterSense.StationArray stationArray, out ISDllBridge.ISD_TRACKER_DATA_TYPE dataISense)
		{
			dataISense = new ISDllBridge.ISD_TRACKER_DATA_TYPE();

			for(int i=0;i<ISDllBridge.ISD_MAX_STATIONS;i++)
			{
				ISDllBridge.ISD_STATION_STATE_TYPE isdStation = new ISDllBridge.ISD_STATION_STATE_TYPE();
				isdStation.Position = new float[3];
				isdStation.Orientation = new float[4];
				dataISense.Station[i] = isdStation;
			}

			//[Feature("EC")]
			//[Requirement("Debugging.Exception/error detection (#625)")]
			if(udp==null)
				//[Requirement("Debugging.Exception/error handling (#630)")]
				return false;

			string str = "ISD_GetData -station ";
			//first, send our request
			int activeCount = 0;	//how many stations are active
			for(int i=0;i<ISDllBridge.ISD_MAX_STATIONS;i++)
			{
				//[Requirement("Operational.Minimum frame rate of 15 fps (#1305)")]
				//[Requirement("Operational.Maximum user input latency of 1 second (#1310)")]
				//[Requirement("Operational.Target frame rate of 60 fps (#1300)")]
				if(!stationArray.isActive(i))
					continue;
				activeCount++;
				if (activeCount > 1)
					str += ",";
				str += (i+1).ToString();
			}
			
			//[Requirement("Operational.Minimum frame rate of 15 fps (#1305)")]
			//[Requirement("Operational.Maximum user input latency of 1 second (#1310)")]
			//[Requirement("Operational.Target frame rate of 60 fps (#1300)")]
			if(activeCount<1)
				//[Requirement("Debugging.Exception/error handling (#630)")]
				return true;

			//should have our proper request string now
			//need to turn it into bytes (there must be a better way to do this in C#)
			byte[] bytes = new byte[str.Length];
			for(int i=0;i<str.Length;i++)
				bytes[i] = Convert.ToByte(str[i]);

			//send the command to get 6DOF information from the server (the server actually communicates with the device)
			//[Feature("EC")]
            //[Requirement("Debugging.Exception/error detection (#625)")]
			try
			{
				udp.Send(bytes,bytes.Length);
			}
            //:block[Feature("EH")]
            //:block[Requirement("Debugging.Exception/error handling (#630)")]
			catch(Exception ex)
			{
				HandleException(ex);
				return false;
			}

			//now read it back until we fill up everything
			//get the bytes
			//[Feature("EC")]
            //[Requirement("Debugging.Exception/error detection (#625)")]
			try
			{
				const int timeout = 1; // secs
				//[Feature("EC")]
				//[Requirement("Debugging.Exception/error detection (#625)")]
				if (!udp.PollForData(timeout))
				{
					//[Feature("Logging")]
					//[Requirement("Debugging.Exception/error handling (#630)")]
					//[Requirement("Debugging.Logging (#600)")]
					MyTrace.Log(TraceLevel.Warning,
						"Timed out waiting for station data. (Timeout: {0} secs).",
						timeout);
					//[Requirement("Debugging.Exception/error handling (#630)")]
					return false;
				}

				bytes = udp.Receive(ref sourcePoint);
			}
            //:block[Feature("EH")]
            //:block[Requirement("Debugging.Exception/error handling (#630)")]
			catch(Exception ex)
			{
				HandleException(ex);
				return false;
			}

			//parse it
			str = Encoding.ASCII.GetString(bytes);

			string[] stationMsgs = str.Split(splitSemicolon);

			//[Feature("Logging")]
			//[Requirement("Debugging.Exception/error detection (#625)")]
			if (stationMsgs.Length != activeCount)
			{
				//[Requirement("Debugging.Exception/error handling (#630)")]
				//[Requirement("Debugging.Logging (#600)")]
				MyTrace.Log(TraceLevel.Warning,
					"Did not receive data for all stations. Requested {0} Received {1}",
					activeCount, stationMsgs.Length);
			}

			foreach(string stationMsg in stationMsgs)
			{
				string[] toks = stationMsg.Split(splitSpace);
			
				int station = Int32.Parse(toks[0].Split(splitEquals)[1])-1;

				dataISense.Station[station].Position[0] = (float)Double.Parse(toks[1].Split(splitEquals)[1]);
				dataISense.Station[station].Position[1] = (float)Double.Parse(toks[2].Split(splitEquals)[1]);
				dataISense.Station[station].Position[2] = (float)Double.Parse(toks[3].Split(splitEquals)[1]);
				
				dataISense.Station[station].Orientation[0] = (float)Double.Parse(toks[4].Split(splitEquals)[1]);
				dataISense.Station[station].Orientation[1] = (float)Double.Parse(toks[5].Split(splitEquals)[1]);
				dataISense.Station[station].Orientation[2] = (float)Double.Parse(toks[6].Split(splitEquals)[1]);
			}

			//[Requirement("Debugging.Exception/error handling (#630)")]
			hasGottenData = true;

			//[Requirement("Debugging.Exception/error handling (#630)")]
			return true;
		}

		[Feature("EH")]
		[Requirement("Debugging.Exception/error handling (#630)")]
		void HandleException(Exception ex)
		{
			//[Feature("Logging")]
			if (ex is SocketException)
			{
				SocketException sex = (SocketException) ex;

				// The help docs say that ex.ErrorCode will be the socket error code
				// but its wrong. The value should be WSAECONNRESET but instead its E_FAIL.
				if (sex.Message.IndexOf("forcibly") > -1)
				{
					if (hasGottenData)
					{
						// ISenseServer died (CTRL^C)
						//[Requirement("Debugging.Logging (#600)")]
						//[Requirement("Debugging.Exception/error handling (#630)")]
						MyTrace.Log(TraceLevel.Warning, 
							"InterSenseSocket::GetData() - ISenseServer '{0}' appears to have died.",
							hostname);
					}
					else
					{
						// ISenseServer died (CTRL^C)
						//[Requirement("Debugging.Exception/error handling (#630)")]
						//[Requirement("Debugging.Logging (#600)")]
						MyTrace.Log(TraceLevel.Warning, 
							"InterSenseSocket::GetData() - ISenseServer not running on '{0}'.",
							hostname);
					}
				}
				else
				{
					//[Requirement("Debugging.Exception/error handling (#630)")]
					//[Requirement("Debugging.Logging (#600)")]
					MyTrace.Log(TraceLevel.Info, "InterSenseSocket::GetData() - Socket for '(0}' was closed: {1}",
						hostname,
						sex.ToString());
				}
			}
			else        
			{
				//[Requirement("Debugging.Exception/error handling (#630)")]
				//[Requirement("Debugging.Logging (#600)")]
				MyTrace.Log(TraceLevel.Error, "InterSenseSocket::GetData() - Socket error for host '{0}': {1}",
					hostname,
					ex.ToString());
			}

			//[Requirement("Debugging.Exception/error handling (#630)")]
			//[Requirement("Nonfunctional.Clean shutdown (#1405)")]
			Close();
		}

		/*
		 * member data
		 * */
		protected	MyUdpClient	udp;			//this is what we use to send and receive data (has destination data in it)
		protected	IPEndPoint	sourcePoint;	//this identifies where we get data from
		protected	string		hostname;		//Keep track of original hostname for error msgs
		
		[Requirement("Debugging.Exception/error handling (#630)")]
		protected	bool		hasGottenData = false;
		
		[Feature("Opt")]
		[Requirement("Operational.Minimum frame rate of 15 fps (#1305)")]
		[Requirement("Operational.Maximum user input latency of 1 second (#1310)")]
		[Requirement("Operational.Target frame rate of 60 fps (#1300)")]
		char[]		splitEquals = "=".ToCharArray();

		[Feature("Opt")]
		[Requirement("Operational.Minimum frame rate of 15 fps (#1305)")]
		[Requirement("Operational.Maximum user input latency of 1 second (#1310)")]
		[Requirement("Operational.Target frame rate of 60 fps (#1300)")]
		char[]		splitSpace = " ".ToCharArray();

		[Feature("Opt")]
		[Requirement("Operational.Minimum frame rate of 15 fps (#1305)")]
		[Requirement("Operational.Maximum user input latency of 1 second (#1310)")]
		[Requirement("Operational.Target frame rate of 60 fps (#1300)")]
		char[]		splitSemicolon = ";".ToCharArray();
	}
}
