///////////////////////////////////////////////////////////////////////
//  Copyright (C) 2005 Columbia University.  All Rights Reserved.
//
//      $Header: /Goblin/Engine/Devices/InterSense/InterSenseStation.cs 13    2/02/07 5:18a Eaddy $
//
//    File name:	InterSenseStation.cs
//   Created by:	Marc Eaddy
//   Created on:	02/18/2005
//
//      Purpose:	
//
//  Description:	
//
///////////////////////////////////////////////////////////////////////
/**
 * $Log: /Goblin/Engine/Devices/InterSense/InterSenseStation.cs $
 * 
 * 13    2/02/07 5:18a Eaddy
 * o Final assignment used for ASOC submission
 * 
 * 12    1/12/07 2:23p Eaddy
 * o After annotating all statements with their corresponding requirements
 * 
 * 11    9/29/06 6:18p Eaddy
 * o Annotated program elements with the concern they realize
 * 
 * 10    5/28/05 1:21p Eaddy
 * Changed copyright
 * 
 * 9     5/07/05 12:49p Eaddy
 * o Removed commented code
 * 
 * 8     4/20/05 3:24p Edp2002
 * Orientation of the tracker was SOOOO daft ... and now i fixed it
 * 
 * 7     4/08/05 1:44p Jrw2005
 * 
 * 6     4/05/05 5:56p Eaddy
 * o Fixed comment
 * 
 * 5     4/01/05 5:29p Eaddy
 * o Fixed transformation
 * 
 * 4     2/22/05 11:46p Eaddy
 * 
 * 3     2/22/05 11:24p Eaddy
 * o Now uses lightweight C# delegates instead of Java-style interfaces
 * 
 * 2     2/22/05 4:09p Eaddy
 * o Fixed head-eye xfrm (sign for y component was wrong)
 * 
 * 1     2/18/05 9:59p Eaddy
 * o Created a generic 6DOF controller class that supports 6DOF data from
 * a DX camera or an IS900 tracker
 */
using Phx.Aop.Attributes;

using System;
using System.Diagnostics;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

using System.Xml.Serialization;

namespace Goblin.Engine.Devices
{
	/// <summary>
	/// Summary description for InterSenseStation.
	/// </summary>
	[Feature("Input")]
	[Requirement("Device.Position and orientation trackers (#150)")]
	[Requirement("Device.Local device state (#125)")]
	[Requirement("Device.Device abstraction (#100)")]
	[Requirement("Device.Head mounted displays (#195)")]
	[Requirement("3D UI.Optical see-through HMD support (#910)")]
	[Requirement("Device.Networked input devices and trackers (#290)")]
	[Requirement("Device.Network-level device integration (#240)")]
    [Requirement("Rendering.Coordinate system independence (#715)")]
    [Requirement("Rendering.Multiple coordinate systems (#705)")]
    [Requirement("Rendering.Relative coordinate systems (#710)")]
	public class InterSenseStation : InputDevice_6DOF
	{
		public long nStationIndex = -1;

		//the head transform is: (0, -0.1397, -0.1524), as x,y,z in intersense format.
		// We make it DX format
		public Matrix matTRANSFORM_TRACKER2EYE = Matrix.Identity;
		//NOTE: removed readonly above, so it could be serialized
		//also removed static, so each tracker can have its own transform (seems possible)

		private Matrix mat = Matrix.Identity;

		private ISDllBridge.ISD_STATION_STATE_TYPE	state;

		public InterSenseStation(long _nStationIndex, Matrix tracker_to_eye_transform)
		{
			nStationIndex = _nStationIndex;
			matTRANSFORM_TRACKER2EYE = tracker_to_eye_transform;
		}

		public void SetData(ISDllBridge.ISD_TRACKER_DATA_TYPE dataISense)
		{
            //[Requirement("Debugging.Exception/error detection (#625)")]
            //[Requirement("Debugging.Exception/error handling (#630)")]
			Debug.Assert(nStationIndex != -1);
			state = dataISense.Station[nStationIndex];
			CreateDXTransformationMatrix();
		}
		
		// Position[0],[1], and [3] correspond to x, y, and z,
		// so you might expect Orientation[0], [1], and [2] to
		// correspond to rotation around the x, y, and z axes.  However,
		// InterSense appears to prefer the order implied by the traditional
		// order of "yaw,pitch,roll".  Orientation[2] is actually "yaw"
		// (rotation around z-axis), Orientation[1] is "pitch" (rotation around
		// y-axis) and Orientation[0] is actually "roll" (rotation around x-axis).
		private void CreateDXTransformationMatrix()
		{
			//Trace.WriteLine(string.Format("ISENSE xrot:{0,4:F}, yrot:{1,4:F}, zrot:{2,4:F}",
			//	aryOrientation[2], aryOrientation[1], aryOrientation[0]));

			// Convert InterSense position and orientation into a transformation
			// matrix appropriate for DirectX.  InterSense is a right-hand coordinate
			// system, DirectX is a left-hand coordinate system.
			//
			// InterSense:    DirectX:		Translation:  Rotation:
			//        ^         y^  ^ z     dx = -ix      
			//   X   / y         | /        dy = -iz
			//  <---/            |---->     dz =  iy
			//      |               x
			//    z v

			mat = 
				Matrix.RotationZ(Geometry.DegreeToRadian(-state.Orientation[2])) *
				Matrix.RotationX(Geometry.DegreeToRadian(-state.Orientation[1])) *
				Matrix.RotationY(Geometry.DegreeToRadian(-90 + state.Orientation[0])) *
				Matrix.Translation(-state.Position[0], -state.Position[2], state.Position[1]) *
				matTRANSFORM_TRACKER2EYE;

			//Trace.Write(SceneGraph.Matrix2String(mat, "mat"));
		}

		override public Matrix GetTransform()
		{
			return mat;
		}
	}
}
