///////////////////////////////////////////////////////////////////////
//  Copyright (C) 2005 Columbia University.  All Rights Reserved.
//
//      $Header: /Goblin/Engine/Scene/MyMeshContainer.cs 47    2/02/07 5:18a Eaddy $
//
//    File name:	MyMeshContainer.cs
//   Created by:	Marc Eaddy
//   Created on:	02/02/2005
//
//      Purpose:	
//
//  Description:	
//
///////////////////////////////////////////////////////////////////////
/**
 * $Log: /Goblin/Engine/Scene/MyMeshContainer.cs $
 * 
 * 47    2/02/07 5:18a Eaddy
 * o Final assignment used for ASOC submission
 * 
 * 46    1/19/07 5:32p Eaddy
 * o Added Requirement annotations
 * 
 * 45    1/12/07 2:23p Eaddy
 * o After annotating all statements with their corresponding requirements
 * 
 * 44    10/10/06 3:15p Eaddy
 * o Removed redundant concern annotations
 * o We now have 100% concern annotation coverage
 * 
 * 43    9/29/06 6:18p Eaddy
 * o Annotated program elements with the concern they realize
 * 
 * 42    5/02/06 11:03a Eaddy
 * o Moved MyTrace into separate library
 * 
 * 41    11/11/05 5:45p Eaddy
 * o Can now easily set the default color or material
 * o Fixed bug when doing AR rendering where objects were still being
 * shaded.  However, chroma keying still has a problem because "real"
 * objects have halos due to anti-aliasing.  We just use black because its
 * halos are less noticeable.  How do you turn off anti-aliasing?
 * 
 * 40    5/31/05 5:53p Eaddy
 * o Minor
 * 
 * 39    5/31/05 2:57p Eaddy
 * o Fixed rendering of bounding vol
 * 
 * 38    5/30/05 11:51p Eaddy
 * o Fixed bug that was introduced when I added the walls
 * 
 * 37    5/28/05 8:06p Eaddy
 * o Now all trace statements go thru MyTrace for consistency
 * 
 * 36    5/28/05 1:21p Eaddy
 * Changed copyright
 * 
 * 35    5/25/05 6:32p Eaddy
 * o Removed BSP stuff
 * 
 * 34    5/17/05 10:46p Eaddy
 * o Moved Bounds class into separate file
 * 
 * 33    5/17/05 3:32a Eaddy
 * o Trying to fix robot AI
 * 
 * 32    5/17/05 2:03a Eaddy
 * o Optimization: getTransformedVertices() now also uses unsafe raw
 * buffer access
 * 
 * 31    5/17/05 1:48a Eaddy
 * o Simplified method
 * 
 * 30    5/17/05 12:17a Eaddy
 * o Optimization: Separated out the bounds info into Bounds class. This
 * allows us to query
 * for bounds info without accessing MyMeshCont which causes extra handles
 * to be
 * created and GC'd
 * 
 * 29    5/16/05 11:03p Eaddy
 * o Optimized FrameMove by accessing raw buffer via unsafe pointer
 * 
 * 28    5/16/05 5:11p Eaddy
 * o Optimized obtaining the transformed vertices. Only does it for nodes
 * that
 * need fine grained collision detection.
 * 
 * 27    5/16/05 4:36p Eaddy
 * o Removed unused function
 * 
 * 26    5/16/05 4:33p Eaddy
 * o Removed commented code
 * 
 * 25    5/16/05 4:33p Eaddy
 * o Removed ToString()
 * 
 * 24    5/16/05 4:32p Eaddy
 * o Optimization: Uses GraphicsStream
 * 
 * 23    5/13/05 7:28p Edp2002
 * FineGrainCD
 * 
 * 22    5/13/05 7:17p Edp2002
 * BS fixed
 * 
 * 21    5/13/05 6:35p Edp2002
 * AABB and BS and NO MESH
 * 
 * 20    5/13/05 5:47p Eaddy
 * o Now only clones the mesh once
 * 
 * 19    5/13/05 4:56p Eaddy
 * o Added comment
 * 
 * 18    5/06/05 3:01p Edp2002
 * isColliding? no ... ok ... gone
 * 
 * 17    4/25/05 6:04p Eaddy
 * o Removed transformed vertices
 * 
 * 16    4/25/05 4:25p Eaddy
 * o Trivial changes
 * 
 * 15    4/25/05 4:03p Eaddy
 * o Can now render the mesh as a black object (for augmented reality
 * display)
 * 
 * 14    4/15/05 6:30p Eaddy
 * o Now renders using default material (red) if no materials are
 * specified
 * 
 * 13    4/15/05 3:23p Eaddy
 * o Added ability to add meshes to scene graph
 * 
 * 12    4/08/05 2:14p Eaddy
 * o Optimization: Transformed vertices are cached
 * 
 * 11    4/08/05 12:27p Eaddy
 * o Now uses Geometry.ComputerBoundingSphere
 * 
 * 10    4/06/05 7:00p Eaddy
 * o Optimized collision detection (still slow)
 * 
 * 9     4/05/05 4:30p Edp2002
 * TransMesh works ... collision ... still looking
 * 
 * 8     4/01/05 8:53p Edp2002
 * still unfinished collision detection
 * 
 * 7     4/01/05 7:32p Edp2002
 * unfinished collision detection changes
 * 
 * 6     3/23/05 10:56p Eaddy
 * o More BSP work
 * 
 * 5     2/18/05 8:33p Eaddy
 * o Renamed MyFrame to Node for clarity
 * 
 * 4     2/18/05 7:47p Eaddy
 * o Moved rendering code into MyFrame and MyMeshContainer
 * 
 * 3     2/10/05 4:21p Eaddy
 * o Started to add ToString() method but didn't finish
 */

using Phx.Aop.Attributes;

using System;
using System.Diagnostics;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;
using Microsoft.Samples.DirectX.UtilityToolkit;

using Columbia.Utilities;

using Goblin.Engine.Utilities;

namespace Goblin.Engine.Scene
{
	/// <summary>
	/// The mesh container class that will hold the animation data
	/// </summary>
	[Feature("SceneGraph.Rendering")]
	[Requirement("Rendering.Graphics API integration - Direct3D (#700)")]
	[Requirement("Rendering.Scene graph (#735)")]
	public class MyMeshContainer : MeshContainer, IDisposable
	{
		// Transformed Mesh
		[Feature("SceneGraph.Movement")]
		private Vector3[] origVerts = null;
		[Feature("SceneGraph.Movement")]
		private Mesh transformedMesh = null;

		[Feature("SceneGraph.CollisionDetection")]
		[Requirement("App Development.Collision detection (#1150)")]
		Bounds bounds = new Bounds();

		[Feature("SceneGraph.CollisionDetection")]
		[Requirement("App Development.Collision detection (#1150)")]
		bool fineGCD = false;

		// Array data
		[Feature("SceneGraph.Rendering")]
		private Texture[] meshTextures = null;
		[Feature("SceneGraph.Rendering")]
		private BoneCombination[] bones;
		[Feature("SceneGraph.Rendering")]
		private Matrix[] offsetMatrices;
		[Feature("SceneGraph.Rendering")]
		private Node[] frameMatrices;

		[Feature("GUI.AR")]
		[Requirement("3D UI.Real-world object occlusion (#935)")]
		[Requirement("3D UI.Optical see-through HMD support (#910)")]
		private readonly static Material matBLACK;

		// Instance data
		[Feature("SceneGraph.Rendering")]
		private int numAttributes = 0;
		[Feature("SceneGraph.Rendering")]
		private int numInfluences = 0;
		[Feature("SceneGraph.Rendering")]
		private int numPalette = 0;

		[Feature("GUI.AR")]
		[Requirement("3D UI.Real-world object occlusion (#935)")]
		[Requirement("3D UI.Optical see-through HMD support (#910)")]
		private bool renderAsBlack = false;

		[Feature("SceneGraph.Rendering")]
		public Material matDefault = new Material();

		[Requirement("3D UI.Real-world object occlusion (#935)")]
		[Requirement("3D UI.Optical see-through HMD support (#910)")]
		static MyMeshContainer()
		{
			matBLACK = new Material();
			//matBLACK.Ambient = System.Drawing.Color.FromArgb(0, 255, 64);
			//matBLACK.Diffuse = System.Drawing.Color.FromArgb(0, 255, 64);
			matBLACK.Ambient = System.Drawing.Color.Black;
			matBLACK.Diffuse = System.Drawing.Color.Black;
		}

		public MyMeshContainer()
		{
			matDefault.Ambient = System.Drawing.Color.Red;
			matDefault.Diffuse = System.Drawing.Color.Red;
		}

		[Requirement("Nonfunctional.Clean shutdown (#1405)")]
		public void Dispose()
		{
			//[Requirement("Debugging.Exception/error detection (#625)")]
			if (transformedMesh != null)
			{
				//[Requirement("Nonfunctional.Clean shutdown (#1405)")]
				//[Requirement("Rendering.Graphics API integration - Direct3D (#700)")]
				//[Requirement("Rendering.Scene graph (#735)")]
				transformedMesh.Dispose();
				//[Requirement("Debugging.Exception/error detection (#625)")]
				transformedMesh = null;
			}

			//[Requirement("Debugging.Exception/error detection (#625)")]
			if (bounds != null)
			{
				//[Requirement("Nonfunctional.Clean shutdown (#1405)")]
				//[Requirement("App Development.Collision detection (#1150)")]
				bounds.Dispose();
				//[Requirement("Debugging.Exception/error detection (#625)")]
				bounds = null;
			}
		}

		// Public properties

		//[Requirement("Rendering.Scene graph (#735)")]
		[Requirement("App Development.Object model (#1110)")]
		public MyMeshContainer MyNextContainer
		{
			get { return NextContainer as MyMeshContainer; }
		}

		//[Requirement("Rendering.Graphics API integration - Direct3D (#700)")]
		//[Requirement("Rendering.Scene graph (#735)")]
		//[Requirement("App Development.Object model (#1110)")]
		public Mesh TransformedMesh
		{
			get { return transformedMesh; }
		}

		//[Requirement("Rendering.Graphics API integration - Direct3D (#700)")]
		//[Requirement("Rendering.Scene graph (#735)")]
		//[Requirement("App Development.Object model (#1110)")]
		public Material DefaultMaterial
		{
			get { return matDefault; }
			set { matDefault = value; }
		}

		//[Requirement("Rendering.Scene graph (#735)")]
		//[Requirement("App Development.Object model (#1110)")]
		public System.Drawing.Color DefaultColor
		{
			set 
			{ 
				matDefault.Ambient = value;
				matDefault.Diffuse = value;
			}
		}

		//[Requirement("Rendering.Scene graph (#735)")]
		//[Requirement("App Development.Object model (#1110)")]
		//[Requirement("App Development.Collision detection (#1150)")]
		public Bounds Bounds
		{
			get { return bounds; }
		}

		//[Requirement("Rendering.Scene graph (#735)")]
		//[Requirement("App Development.Object model (#1110)")]
		//[Requirement("App Development.Collision detection (#1150)")]
		public bool FineGrainCD
		{
			get { return fineGCD; }
			set { fineGCD = value; }
		}

		//[Requirement("Rendering.Scene graph (#735)")]
		//[Requirement("App Development.Object model (#1110)")]
		//[Requirement("3D UI.Optical see-through HMD support (#910)")]
		//[Requirement("3D UI.Real-world object occlusion (#935)")]
		public bool RenderAsBlack
		{
			get { return renderAsBlack; }
			set { renderAsBlack = value; }
		}

		/// <summary>Retrieve the textures used for this container</summary>
		[Requirement("Rendering.Scene graph (#735)")]
		[Requirement("Rendering.Graphics API integration - Direct3D (#700)")]
		public Texture[] GetTextures() { return meshTextures; }

		/// <summary>Set the textures used for this container</summary>
		[Requirement("Rendering.Scene graph (#735)")]
		[Requirement("Rendering.Graphics API integration - Direct3D (#700)")]
		public void SetTextures(Texture[] textures) { meshTextures = textures; }
        
		/// <summary>Retrieve the bone combinations used for this container</summary>
		[Requirement("Rendering.Scene graph (#735)")]
		[Requirement("Rendering.Graphics API integration - Direct3D (#700)")]
		public BoneCombination[] GetBones() { return bones; }

		/// <summary>Set the bone combinations used for this container</summary>
		[Requirement("Rendering.Scene graph (#735)")]
		[Requirement("Rendering.Graphics API integration - Direct3D (#700)")]
		public void SetBones(BoneCombination[] b) { bones = b; }
        
		/// <summary>Retrieve the animation frames used for this container</summary>
		[Requirement("Rendering.Scene graph (#735)")]
		[Requirement("Rendering.Graphics API integration - Direct3D (#700)")]
		public Node[] GetFrames() { return frameMatrices; }

		/// <summary>Set the animation frames used for this container</summary>
		[Requirement("Rendering.Scene graph (#735)")]
		[Requirement("Rendering.Graphics API integration - Direct3D (#700)")]
		public void SetFrames(Node[] frames) { frameMatrices = frames; }
        
		/// <summary>Retrieve the offset matrices used for this container</summary>
		[Requirement("Rendering.Scene graph (#735)")]
		[Requirement("Rendering.Graphics API integration - Direct3D (#700)")]
		public Matrix[] GetOffsetMatrices() { return offsetMatrices; }

		/// <summary>Set the offset matrices used for this container</summary>
		[Requirement("Rendering.Scene graph (#735)")]
		[Requirement("Rendering.Graphics API integration - Direct3D (#700)")]
		public void SetOffsetMatrices(Matrix[] matrices) { offsetMatrices = matrices; }
        
		/// <summary>Total number of attributes this mesh container contains</summary>
		[Requirement("Rendering.Scene graph (#735)")]
		[Requirement("Rendering.Graphics API integration - Direct3D (#700)")]
		public int NumberAttributes { get { return numAttributes; } set { numAttributes = value; } }

		/// <summary>Total number of influences this mesh container contains</summary>
		[Requirement("Rendering.Scene graph (#735)")]
		[Requirement("Rendering.Graphics API integration - Direct3D (#700)")]
		public int NumberInfluences { get { return numInfluences; } set { numInfluences = value; } }

		/// <summary>Total number of palette entries this mesh container contains</summary>
		[Requirement("Rendering.Scene graph (#735)")]
		[Requirement("Rendering.Graphics API integration - Direct3D (#700)")]
		public int NumberPaletteEntries { get { return numPalette; } set { numPalette = value; } }

		[Feature("Persistence.Loading")]
		[Requirement("Configuration.Device configuration files (#135)")]
		[Requirement("Configuration.Import - DirectX (#720)")]
		[Requirement("Rendering.Dynamic scene geometry (#740)")]
		[Requirement("Rendering.Graphics API integration - Direct3D (#700)")]
		public void Init(	Device device, 
							string directory, 
							XFileData xfileData, 
							[Requirement("Debugging.Logging (#600)")] string indent)
		{
			GraphicsStream adjacency;
			ExtendedMaterial[] materials;
			EffectInstance[] effects;
			Mesh mesh = Mesh.FromX(xfileData, MeshFlags.Managed,
				device, out adjacency, out materials, out effects);

			Init(device, xfileData.Name, directory, mesh, adjacency, materials, indent);
		}

		[Feature("Persistence.Loading")]
		[Requirement("Configuration.Device configuration files (#135)")]
		[Requirement("Configuration.Import - DirectX (#720)")]
		[Requirement("Rendering.Dynamic scene geometry (#740)")]
		[Requirement("Rendering.Graphics API integration - Direct3D (#700)")]
		public void Init(	Device device,
							string name,
							string directory,
							Mesh mesh, 
							GraphicsStream adjacency, 
							ExtendedMaterial[] materials, 
							[Requirement("Debugging.Logging (#600)")] string indent)
		{
			// Make sure the mesh has normals
			if ((mesh.VertexFormat & VertexFormats.Normal) == 0)
			{
				// Clone the mesh
				Mesh tempMesh = mesh.Clone(mesh.Options.Value, 
					mesh.VertexFormat | VertexFormats.Normal, device);

				// Destroy current mesh, use the new one
				//[Requirement("Nonfunctional.Clean shutdown (#1405)")]
				mesh.Dispose();
				mesh = tempMesh;
				mesh.ComputeNormals();
			}

			// Wrap the mesh in a MeshData object
			MeshData meshData = new MeshData();
			meshData.Mesh = mesh;

			// Initialize the ambient color to be the same as the diffuse color.
			// That way we can see something using just ambient light instead of
			// needing to create a light.  If we create a light then diffuse will
			// get used.
			//[Requirement("Configuration.Device configuration files (#135)")]
			//[Requirement("Configuration.Import - DirectX (#720)")]
			//[Requirement("Rendering.Dynamic scene geometry (#740)")]
			//[Requirement("Rendering.Graphics API integration - Direct3D (#700)")]
			//[Requirement("Debugging.Exception/error detection (#625)")]
			for(int i = 0; materials != null && i < materials.Length; ++i)
			{
				//[Requirement("Configuration.Device configuration files (#135)")]
				//[Requirement("Configuration.Import - DirectX (#720)")]
				//[Requirement("Rendering.Dynamic scene geometry (#740)")]
				//[Requirement("Rendering.Graphics API integration - Direct3D (#700)")]
				{
					Material mat = materials[i].Material3D;
					mat.AmbientColor = mat.DiffuseColor;
	
					// Note: Material is a value type so the update above
					// only updates the local copy!  We have to copy it back
					// manually.
					materials[i].Material3D = mat;
				}
			}

			if (adjacency != null)
			{
				meshData.Mesh.OptimizeInPlace(MeshFlags.OptimizeVertexCache | MeshFlags.OptimizeCompact | 
					MeshFlags.OptimizeAttributeSort, adjacency);
			}

			// Create the mesh container. This contains the mesh and its
			// associated  materials and textures.

			Name = name;
			MeshData = meshData;
			SetMaterials(materials);
			SetAdjacency(adjacency);

			// Add the mesh's textures

			// Trick to find out exactly how many textures are present
			int nTexCount = 
				(int) (mesh.VertexFormat & VertexFormats.TextureCountMask) >>
				(int) VertexFormats.TextureCountShift;

			if (materials != null)
			{
				Texture[] meshTextures = new Texture[materials.Length];

				// Create any textures
				for (int i = 0; i < materials.Length; ++i)
				{
					meshTextures[i] = null;

					if (materials[i].TextureFilename != null)
					{
						string path = MyUtility.MyFindMediaFile(directory, 
							materials[i].TextureFilename);

						//[Requirement("Debugging.Logging (#600)")]
						MyTrace.WriteLine(indent + "  Loading texture: " + path);
						
                        //[Feature("EC")]
                        //[Requirement("Debugging.Exception/error detection (#625)")]
						try
						{
							meshTextures[i] = ResourceCache.GetGlobalInstance().CreateTextureFromFile(
								device, path);
						}
                        //:block[Feature("EH")]
						//:block[Requirement("Debugging.Exception/error handling (#630)")]
						catch (InvalidDataException ex)
						{
						    //[Requirement("Debugging.Logging (#600)")]
						    //[Requirement("Debugging.Exception/error handling (#630)")]
                            {
							    // D3DXERR_INVALIDDATA actually should say "File Not Found"
							    MyTrace.WriteLine("Texture file not found: " + path);
							    MyTrace.WriteLine(ex.StackTrace);
							    // RETURN ERROR???
                            }
						}
					}
				}
					
				SetTextures(meshTextures);
			}

			// Create a normalized version (has specific known vertex format)
			transformedMesh = mesh.Clone(MeshData.Mesh.Options.Value, 
				VertexFormats.Position, device);

			VertexBuffer vb = transformedMesh.VertexBuffer;

			// Copy the original vertices so we can use them as a base for later
			// transformations
			origVerts = (Vector3[]) vb.Lock(0, 
											typeof(Vector3), 
											LockFlags.ReadOnly,
											mesh.NumberVertices);
			vb.Unlock();
		}

		[Feature("SceneGraph.Movement")]
		[Requirement("Rendering.Scene graph (#735)")]
		[Requirement("Rendering.Graphics API integration - Direct3D (#700)")]
		public void FrameMove( Device device, Matrix parentMatrix)
		{
			VertexBuffer vbTrans = transformedMesh.VertexBuffer;

            //[Requirement("Debugging.Exception/error detection (#625)")]
            //[Requirement("Debugging.Exception/error handling (#630)")]
			Debug.Assert(vbTrans.Description.VertexFormat == CustomVertex.PositionOnly.Format);

			GraphicsStream strmDest = vbTrans.Lock(0, 0, LockFlags.None);

			unsafe
			{
				Vector3* pVertsDest = (Vector3*) strmDest.InternalDataPointer;

				for(int i = 0; i < origVerts.Length; ++i)
				{
					pVertsDest[i] = Vector3.TransformCoordinate(origVerts[i], parentMatrix);
				}
			}

			// Update the bounding sphere
			//[Requirement("App Development.Collision detection (#1150)")]
			bounds.update(strmDest, origVerts.Length);

			//[Requirement("Nonfunctional.Clean shutdown (#1405)")]
			{
				strmDest.Dispose();
	
				vbTrans.Unlock();
				vbTrans.Dispose();
			}
		}

		/// <summary>Render a mesh container</summary>
		public void Render(Device device)
		{
			ExtendedMaterial[] materials = GetMaterials();

			if (materials.Length == 0)
			{
				//[Requirement("3D UI.Optical see-through HMD support (#910)")]
				//[Requirement("3D UI.Real-world object occlusion (#935)")]
				device.Material = renderAsBlack ? matBLACK : matDefault;
				MeshData.Mesh.DrawSubset(0);
			}
			else
			{
				//[Requirement("3D UI.Optical see-through HMD support (#910)")]
				//[Requirement("3D UI.Real-world object occlusion (#935)")]
				if (renderAsBlack)
				{
					device.Lights[0].Enabled = false;
					device.Material = matBLACK;
				}

				// Set and draw each of the materials in the mesh
				for (int i = 0; i < materials.Length; ++i)
				{
					// Setup the material and texture
					if (!renderAsBlack)
					{
						device.Material = materials[i].Material3D;

						if (meshTextures != null)
						{
							device.SetTexture(0, meshTextures[i]);
						}
					}

					// Finally draw the subset
					MeshData.Mesh.DrawSubset(i);
				}

				//[Requirement("3D UI.Optical see-through HMD support (#910)")]
				//[Requirement("3D UI.Real-world object occlusion (#935)")]
				if (renderAsBlack)
				{
					device.Lights[0].Enabled = true;
				}
			}
		}

		[Feature("Diagnostics")]
		[Requirement("Debugging.Application visualization (#620)")]
		[Requirement("App Development.Collision detection (#1150)")]
		public void RenderBoundingVolume(Device device)
		{
			bounds.Render(device);
		}

		/// <summary>
		/// An attempt to implement Mesh.DrawSubset() so that we could draw a face
		/// with a different color.
		/// </summary>
		[Requirement("Nonfunctional.Unused (#1400)")]
		public void Render2(Device device)
		{
			ExtendedMaterial[] materials = GetMaterials();

			//MyUtility.TraceMesh(MeshData.Mesh, "blah");

			Mesh tempMesh = MeshData.Mesh.Clone(MeshData.Mesh.Options.Value, 
				VertexFormats.Position | VertexFormats.Normal, device);

			AttributeRange[] attributes = tempMesh.GetAttributeTable();

			IndexBuffer ib = tempMesh.IndexBuffer;
			short[] indices = (short[]) ib.Lock(0, 
				typeof(short), LockFlags.ReadOnly, MeshData.Mesh.NumberFaces*3);

			VertexBuffer vb = tempMesh.VertexBuffer;
            //[Requirement("Debugging.Exception/error detection (#625)")]
            //[Requirement("Debugging.Exception/error handling (#630)")]
			Debug.Assert(vb.Description.VertexFormat == CustomVertex.PositionNormal.Format);

			CustomVertex.PositionNormal[] vertices = 
				(CustomVertex.PositionNormal[]) vb.Lock(0, 
				typeof(CustomVertex.PositionNormal),
				LockFlags.ReadOnly,
				tempMesh.NumberVertices);

			foreach(AttributeRange attr in attributes)
			{
				// Setup the material and texture
				device.Material = materials[attr.AttributeId].Material3D;
				device.SetTexture(0, meshTextures[attr.AttributeId]);
				device.SetStreamSource( 0, vb, 0);
				device.Indices = ib;
				device.VertexFormat = CustomVertex.PositionNormal.Format;
				device.DrawIndexedPrimitives(	PrimitiveType.TriangleList,
												0,
												0, 
												attr.FaceCount * 3,
												attr.FaceStart * 3,
												attr.FaceCount);
			}

			//[Requirement("Nonfunctional.Clean shutdown (#1405)")]
			{
				ib.Unlock();
				ib.Dispose();
	
				vb.Unlock();
				vb.Dispose();
	
				tempMesh.Dispose();
			}
		}

		[Feature("SceneGraph.Movement")]
		[Requirement("Rendering.Scene graph (#735)")]
		public void getTransformedVertices(Matrix mat, out Vector3[] verts)
		{
			getTransformedVertices(origVerts, mat, out verts);
		}

		[Feature("SceneGraph.Movement")]
		[Requirement("Rendering.Scene graph (#735)")]
		void getTransformedVertices(Vector3[] orig, Matrix mat, out Vector3[] verts)
		{
			verts = new Vector3[orig.Length];
			for(int i = 0; i < orig.Length; ++i)
			{
				verts[i] = Vector3.TransformCoordinate(orig[i], mat);
			}
		}

		[Feature("SceneGraph.CollisionDetection")]
		[Requirement("App Development.Collision detection (#1150)")]
		public bool Overlaps(Bounds bounds2)
		{
			if (!Bounds.Overlaps(bounds, bounds2))
				return false;

			// Assumption is that transformedMesh has already been updated
			// during FrameMove()

			VertexBuffer vbTrans = transformedMesh.VertexBuffer;

            //[Requirement("Debugging.Exception/error detection (#625)")]
            //[Requirement("Debugging.Exception/error handling (#630)")]
			Debug.Assert(vbTrans.Description.VertexFormat == CustomVertex.PositionOnly.Format);

			GraphicsStream strmDest = vbTrans.Lock(0, 0, LockFlags.ReadOnly);

			bool overlaps = false;

			unsafe
			{
				Vector3* pVertsDest = (Vector3*) strmDest.InternalDataPointer;

				for(int i = 0; i < origVerts.Length; ++i)
				{
					if (bounds2.Contains(pVertsDest[i]))
					{
						overlaps = true;
						break;
					}
				}
			}

			//[Requirement("Nonfunctional.Clean shutdown (#1405)")]
			{
				strmDest.Dispose();
	
				vbTrans.Unlock();
				vbTrans.Dispose();
			}

			return overlaps;
		}
	}
}
