///////////////////////////////////////////////////////////////////////
//  Copyright (C) 2005 Columbia University.  All Rights Reserved.
//
//      $Header: /Goblin/Engine/Scene/Nodes/CameraNode.cs 12    1/12/07 2:23p Eaddy $
//
//    File name:	CameraNode.cs
//   Created by:	Marc Eaddy
//   Created on:	02/18/2005
//
//      Purpose:	
//
//  Description:	
//
///////////////////////////////////////////////////////////////////////
/**
 * $Log: /Goblin/Engine/Scene/Nodes/CameraNode.cs $
 * 
 * 12    1/12/07 2:23p Eaddy
 * o After annotating all statements with their corresponding requirements
 * 
 * 11    9/29/06 6:18p Eaddy
 * o Annotated program elements with the concern they realize
 * 
 * 10    5/02/06 11:03a Eaddy
 * o Moved MyTrace into separate library
 * 
 * 9     5/31/05 5:46p Eaddy
 * o Added ability to filter which nodes are rendered using a string
 * 
 * 8     5/28/05 1:21p Eaddy
 * Changed copyright
 * 
 * 7     4/25/05 6:04p Eaddy
 * o Cleanup
 * 
 * 6     4/25/05 5:51p Eaddy
 * o Trivial
 * 
 * 5     4/22/05 7:53p Edp2002
 * We now have a collidable damagable body for the hero ... a cylinder
 * 
 * 4     4/22/05 6:43p Eaddy
 * o Now you can get the position of the camera (before it was inverted --
 * the lookat position)
 * 
 * 3     4/20/05 6:05p Edp2002
 * CAMERA_PASS added
 * 
 * 2     4/15/05 3:16p Edp2002
 * new Ghetto Collision detection ... and 2 passes
 * 
 * 1     2/18/05 8:55p Eaddy
 * o Initial checkin
 */

using Phx.Aop.Attributes;

using System;

using Microsoft.DirectX;			// Matrix
using Microsoft.DirectX.Direct3D;

using Columbia.Utilities;

using Goblin.Engine.Utilities;

namespace Goblin.Engine.Scene
{
	/// <summary>
	/// Summary description for CameraNode.
	/// </summary>
	[Feature("GUI.Camera")]
	[Requirement("Device.Single monitor displays (#165)")]
	[Requirement("Device.Multi-monitor displays (#190)")]
	[Requirement("Device.Head mounted displays (#195)")]
	[Requirement("3D UI.Optical see-through HMD support (#910)")]
	[Requirement("3D UI.Monoscopic display support (#915)")]
	[Requirement("Rendering.Multiple coordinate systems (#705)")]
	[Requirement("Rendering.Relative coordinate systems (#710)")]
	[Requirement("Rendering.Scene graph (#735)")]
	public class CameraNode : Node
	{
		public CameraNode(string name)
			: base(name)
		{ }

		/// <summary>Pass only the pos as the parentMatrix</summary>
		public override void FrameMove(Node root, Device device, Matrix parentMatrix, float elapsedTime)
		{
			if (Animated)                                                               
			{                                                                           
				SetCombinedTransformations(root, device, Matrix.Identity, elapsedTime); 
			}                                                                           

			Matrix posParent = Matrix.Identity;                                         
			posParent.Translate(MyUtility.MatrixGetTranslation(CombinedTransformationMatrix));

			Node firstChild = (Node) FrameFirstChild;                                   
			if (firstChild != null)                                                     
			{                                                                           
				firstChild.FrameMove(root, device, posParent, elapsedTime);             
				/*
				Node sibling = (Node) firstChild.FrameSibling;
				while ( sibling != null ) 
				{
					sibling.FrameMove(root, device, parentMatrix, elapsedTime);
					sibling = (Node) sibling.FrameSibling;
				}
				*/
			}                                                                           
		}                                                                               

		/// <summary>Draw a frame and all child and sibling frames</summary>
		public override void Render(Device device, string filter, int pass)
		{                                                                               
			if (pass == SceneGraph.CAMERA_PASS)                                        
			{                                                                          
				Matrix matView = CombinedTransformationMatrix;                         
				matView.Invert();                                                      
				//[Requirement("Device.Single monitor displays (#165)")]
				//[Requirement("Device.Multi-monitor displays (#190)")]
				//[Requirement("Device.Head mounted displays (#195)")]
				//[Requirement("3D UI.Optical see-through HMD support (#910)")]
				//[Requirement("3D UI.Monoscopic display support (#915)")]
				//[Requirement("Rendering.Graphics API integration - Direct3D (#700)")]
				//[Requirement("Rendering.Multiple coordinate systems (#705)")]
				//[Requirement("Rendering.Relative coordinate systems (#710)")]
				//[Requirement("Rendering.Scene graph (#735)")]
				device.Transform.View = matView;                                       
			}                                                                          
			else if (pass == SceneGraph.GEOMETRY_PASS)                                 
			{                                                                          
				base.Render(device, filter, pass);                                     
			}                                                                          
		}                                                                              
	}
}
