///////////////////////////////////////////////////////////////////////
//  Copyright (C) 2005 Columbia University.  All Rights Reserved.
//
//      $Header: /Goblin/Engine/Scene/Transformers/AccelerationTransformer.cs 9     1/12/07 2:23p Eaddy $
//
//    File name:	AccelerationTransformer.cs
//   Created by:	Erik Peterson
//   Created on:	04/18/2005
//
//      Purpose:	
//
//  Description:	
//
///////////////////////////////////////////////////////////////////////
/**
 * $Log: /Goblin/Engine/Scene/Transformers/AccelerationTransformer.cs $
 * 
 * 9     1/12/07 2:23p Eaddy
 * o After annotating all statements with their corresponding requirements
 * 
 * 8     10/10/06 3:15p Eaddy
 * o Removed redundant concern annotations
 * o We now have 100% concern annotation coverage
 * 
 * 7     9/29/06 6:18p Eaddy
 * o Annotated program elements with the concern they realize
 * 
 * 6     5/28/05 1:21p Eaddy
 * Changed copyright
 * 
 * 5     5/17/05 3:32a Eaddy
 * o Trying to fix robot AI
 * 
 * 4     5/16/05 3:16p Eaddy
 * o Uses Vector.Empty (optimization?)
 * 
 * 3     5/13/05 3:18p Eaddy
 * o Can now turn collision detection off
 * 
 * 2     5/10/05 8:50a Eaddy
 * o Added Log to comment header
 */

using Phx.Aop.Attributes;

using System;

using System.Diagnostics;			// Debug.Assert()

using Microsoft.DirectX;			// Matrix
using Microsoft.DirectX.Direct3D;	// IntersectionInformation

namespace Goblin.Engine.Scene
{
	/// <summary>
	/// Moves an object by moving it forward in the direction it is pointing
	/// </summary>
	[Feature("SceneGraph.Movement")]
	[Feature("SceneGraph.CollisionDetection")]
	[Requirement("App Development.Collision detection (#1150)")]
	[Requirement("Rendering.Scene graph (#735)")]
	[Requirement("App Development.Simple physics (#1155)")]
	public class AccelerationTransformer : Transformer, ICollisionListener
	{
		Vector3 velocity;
		Vector3 acceleration;

		public AccelerationTransformer([Requirement("App Development.Collision detection (#1150)")]
		                               bool bStopOnCollision)
		{ 
			velocity = Vector3.Empty;
			acceleration = Vector3.Empty;

			//[Feature("SceneGraph.CollisionDetection")]
			//[Requirement("App Development.Collision detection (#1150)")]
			if (bStopOnCollision)
				AddCollisionListener(this);
		}

		public Vector3 Velocity
		{
			get { return velocity; }
			set { velocity = value; }
		}

		public Vector3 Acceleration
		{
			get { return acceleration; }
			set { acceleration = value; }
		}

		public override bool GetDesiredPosition(Xfrm cur, float elapsed, ref Xfrm desired)
		{
			if (velocity == Vector3.Empty && acceleration == Vector3.Empty)
				//[Requirement("Operational.Minimum frame rate of 15 fps (#1305)")]
				//[Requirement("Operational.Maximum user input latency of 1 second (#1310)")]
				//[Requirement("Operational.Target frame rate of 60 fps (#1300)")]
				return false;

			desired = cur.Copy();
			desired.position = cur.position * Matrix.Translation(velocity * elapsed);

			// Update the velocity
			velocity += acceleration * elapsed;

			//[Requirement("Operational.Minimum frame rate of 15 fps (#1305)")]
			//[Requirement("Operational.Maximum user input latency of 1 second (#1310)")]
			//[Requirement("Operational.Target frame rate of 60 fps (#1300)")]
			return true;
		}

		[Feature("SceneGraph.CollisionDetection")]
		[Requirement("App Development.Collision detection (#1150)")]
		public void OnBeforeCollided(	Node node, 
										Node root,
										Node nodeHit,
										Xfrm old, 
										IntersectInformation intersect,
										Xfrm desired)
		{
			// Do nothing
		}

		[Feature("SceneGraph.CollisionDetection")]
		[Requirement("App Development.Collision detection (#1150)")]
		public void OnCollided(	Node node, 
								Node root,
								Node nodeHit,
								Xfrm old, 
								Xfrm collided,
								IntersectInformation intersect,
								ref Xfrm desired)
		{
			velocity = Vector3.Empty;
			acceleration = Vector3.Empty;
		}
	}
}
