///////////////////////////////////////////////////////////////////////
//  Copyright (C) 2005 Columbia University.  All Rights Reserved.
//
//      $Header: /Goblin/Engine/Scene/Transformers/DXCameraTransformer.cs 6     1/12/07 2:23p Eaddy $
//
//    File name:	DXCameraTransformer.cs
//   Created by:	Marc Eaddy
//   Created on:	02/18/2005
//
//      Purpose:	
//
//  Description:	
//
///////////////////////////////////////////////////////////////////////
/**
 * $Log: /Goblin/Engine/Scene/Transformers/DXCameraTransformer.cs $
 * 
 * 6     1/12/07 2:23p Eaddy
 * o After annotating all statements with their corresponding requirements
 * 
 * 5     9/29/06 6:18p Eaddy
 * o Annotated program elements with the concern they realize
 * 
 * 4     5/28/05 1:21p Eaddy
 * Changed copyright
 * 
 * 3     4/25/05 4:42p Eaddy
 * o Transformer no longer stores the Node. This eliminates an unnecessary
 * circular dependency
 * between Node and Transformer, allows Transformers to be reused, and
 * simplifies the design.
 * 
 * 2     4/22/05 6:43p Eaddy
 * o Now you can get the position of the camera (before it was inverted --
 * the lookat position)
 * 
 * 1     4/22/05 5:13p Eaddy
 * o Rearranged things
 * 
 * 3     2/22/05 11:29p Eaddy
 * o Renamed
 * 
 * 2     2/22/05 11:24p Eaddy
 * o Now uses lightweight C# delegates instead of Java-style interfaces
 * 
 * 1     2/18/05 9:59p Eaddy
 * o Created a generic 6DOF controller class that supports 6DOF data from
 * a DX camera or an IS900 tracker
 */
using Phx.Aop.Attributes;

using System;
using Microsoft.DirectX;

using Microsoft.Samples.DirectX.UtilityToolkit;

namespace Goblin.Engine.Scene
{
	/// <summary>
	/// Summary description for DXCameraController.
	/// </summary>
	[Feature("GUI.Camera")]
	[Requirement("Device.Single monitor displays (#165)")]
	[Requirement("Device.Multi-monitor displays (#190)")]
	[Requirement("Rendering.Graphics API integration - Direct3D (#700)")]
	[Requirement("3D UI.Monoscopic display support (#915)")]
	public class DXCameraTransformer : Transformer
	{
		private Camera camera;

		public DXCameraTransformer(Camera _camera)
		{
			camera = _camera;
		}

		public override bool GetDesiredPosition(Xfrm cur, float elapsed, ref Xfrm desired)
		{
			// Note: The device transformation provides rotation as well as
			// position which we are storing in the "position" matrix.
			desired = cur.Copy();
			desired.position = camera.ViewMatrix;
			desired.position.Invert();

			//[Requirement("Operational.Minimum frame rate of 15 fps (#1305)")]
			//[Requirement("Operational.Maximum user input latency of 1 second (#1310)")]
			//[Requirement("Operational.Target frame rate of 60 fps (#1300)")]
			return cur != desired;
		}
	}
}
