///////////////////////////////////////////////////////////////////////
//  Copyright (C) 2005 Columbia University.  All Rights Reserved.
//
//      $Header: /Goblin/Engine/Scene/Transformers/TrackerTransformer.cs 5     1/12/07 2:23p Eaddy $
//
//    File name:	TrackerTransformer.cs
//   Created by:	Marc Eaddy
//   Created on:	04/18/2005
//
//      Purpose:	
//
//  Description:	
//
///////////////////////////////////////////////////////////////////////
/**
 * $Log: /Goblin/Engine/Scene/Transformers/TrackerTransformer.cs $
 * 
 * 5     1/12/07 2:23p Eaddy
 * o After annotating all statements with their corresponding requirements
 * 
 * 4     9/29/06 6:18p Eaddy
 * o Annotated program elements with the concern they realize
 * 
 * 3     5/28/05 1:21p Eaddy
 * Changed copyright
 * 
 * 2     4/25/05 4:42p Eaddy
 * o Transformer no longer stores the Node. This eliminates an unnecessary
 * circular dependency
 * between Node and Transformer, allows Transformers to be reused, and
 * simplifies the design.
 * 
 * 1     4/22/05 4:59p Eaddy
 * o Broke out Transformer and CollisionListener classes into separate
 * files
 */

using Phx.Aop.Attributes;

using System;

using System.Diagnostics;			// Debug.Assert()

using Microsoft.DirectX;			// Matrix
using Microsoft.DirectX.Direct3D;	// IntersectionInformation

using Goblin.Engine.Devices;

namespace Goblin.Engine.Scene
{
	/// <summary>
	/// </summary>
	[Feature("SceneGraph.Movement")]
	[Requirement("Device.Head mounted displays (#195)")]
	[Requirement("Device.Position and orientation trackers (#150)")]
	[Requirement("Device.Device abstraction (#100)")]
	[Requirement("3D UI.Optical see-through HMD support (#910)")]
	public class TrackerTransformer : Transformer
	{
		InputDevice_6DOF device;

		public TrackerTransformer(InputDevice_6DOF device)
		{ 
			this.device = device;
		}

		public override bool GetDesiredPosition(Xfrm cur, float elapsed, ref Xfrm desired)
		{
			// Note: The device transformation provides rotation as well as
			// position which we are storing in the "position" matrix.
			desired = cur.Copy();
			desired.position = device.GetTransform();

			//[Requirement("Operational.Minimum frame rate of 15 fps (#1305)")]
			//[Requirement("Operational.Maximum user input latency of 1 second (#1310)")]
			//[Requirement("Operational.Target frame rate of 60 fps (#1300)")]
			return cur != desired;
		}
	}
}
