///////////////////////////////////////////////////////////////////////
//  Copyright (C) 2005 Columbia University.  All Rights Reserved.
//
//      $Header: /Goblin/Engine/Scene/Transformers/Transformer.cs 25    1/12/07 2:23p Eaddy $
//
//    File name:	Transformer.cs
//   Created by:	Marc Eaddy
//   Created on:	04/18/2005
//
//      Purpose:	
//
//  Description:	
//
///////////////////////////////////////////////////////////////////////
/**
 * $Log: /Goblin/Engine/Scene/Transformers/Transformer.cs $
 * 
 * 25    1/12/07 2:23p Eaddy
 * o After annotating all statements with their corresponding requirements
 * 
 * 24    9/29/06 6:18p Eaddy
 * o Annotated program elements with the concern they realize
 * 
 * 23    6/02/05 9:23p Eaddy
 * o Changed Property to direct member access (makes code cleaner)
 * 
 * 22    5/30/05 11:51p Eaddy
 * o Fixed bug that was introduced when I added the walls
 * 
 * 21    5/28/05 1:21p Eaddy
 * Changed copyright
 * 
 * 20    5/17/05 10:09p Eaddy
 * o Added flag for stopping at the first collision. This isn't working
 * properly since IsCollision no longer obtains
 * the collision distance in every case.
 * 
 * 19    5/17/05 3:32a Eaddy
 * o Trying to fix robot AI
 * 
 * 18    5/16/05 5:11p Eaddy
 * o Optimized obtaining the transformed vertices. Only does it for nodes
 * that
 * need fine grained collision detection.
 * 
 * 17    5/16/05 4:22p Eaddy
 * o Small optimization
 * 
 * 16    4/25/05 4:42p Eaddy
 * o Transformer no longer stores the Node. This eliminates an unnecessary
 * circular dependency
 * between Node and Transformer, allows Transformers to be reused, and
 * simplifies the design.
 * 
 * 15    4/22/05 4:59p Eaddy
 * o Broke out Transformer and CollisionListener classes into separate
 * files
 * 
 * 14    4/22/05 4:19p Jrw2005
 * 
 * 13    4/22/05 3:43p Eaddy
 * o Got rid of framesUntilCollision. You have to use closestHit.Dist and
 * figure the frames out yourself
 * (its more general this way).
 * 
 * 12    4/22/05 3:40p Eaddy
 * o Fixed bug where OnBeforeCollided was never called
 * 
 * 11    4/22/05 3:36p Eaddy
 * o Added OnBeforeCollided
 * 
 * 10    4/20/05 8:36p Edp2002
 * fixes for boltTraker problems ... see email marc
 * 
 * 9     4/20/05 7:57p Edp2002
 * Liger now dies ... 
 * 
 * 8     4/20/05 7:22p Edp2002
 * 
 * 7     4/20/05 7:19p Edp2002
 * 
 * 6     4/20/05 5:23p Eaddy
 * o If no collision listeners are present, the default it to ignore
 * collisions
 * o Renamed class
 * o Fixed bug in AdapterTransformer that caused the camera transformer to
 * be incorrect
 * 
 * 5     4/20/05 5:00p Eaddy
 * o Now collision sticker is used for velocity transformer collision
 * 
 * 4     4/20/05 4:56p Eaddy
 * o Now bolt collision works
 * 
 * 3     4/20/05 3:50p Edp2002
 * added nodeController functions
 * 
 * 2     4/20/05 3:37p Eaddy
 * o Almost working
 * 
 * 1     4/18/05 4:37p Eaddy
 * o Initial checkin
 */

using Phx.Aop.Attributes;

using System;

using System.Diagnostics;			// Debug.Assert()
using System.Collections;			// ArrayList

using Microsoft.DirectX;			// Matrix
using Microsoft.DirectX.Direct3D;	// IntersectionInformation

namespace Goblin.Engine.Scene
{
	/// <summary>
	/// Controls the movement of an object in the scene over time.
	/// Subclasses will define specific 3D movement paths, e.g.,
	/// movement in a straight line.
	/// </summary>
	[Feature("SceneGraph.Movement")]
	[Requirement("Rendering.Scene graph (#735)")]
	public class Transformer
	{
		public Xfrm current;

		ArrayList listeners = new ArrayList();

		public Transformer()
		{
			current = new Xfrm();
		}

		[Feature("SceneGraph.CollisionDetection")]
		[Requirement("App Development.Collision detection (#1150)")]
		[Requirement("Rendering.Scene graph (#735)")]
		public void AddCollisionListener(ICollisionListener listener)
		{
            //[Requirement("Debugging.Exception/error detection (#625)")]
            //[Requirement("Debugging.Exception/error handling (#630)")]
			Debug.Assert(listener != null);
			listeners.Add(listener);
		}

		/// <summary>
		/// Determines the new position for an object.  If arriving
		/// at the new position would cause a collision, 
		/// OnAfterCollisionDetection is called.
		/// </summary>
		/// <param name="root">Root of the scene graph</param>
		public virtual void FrameMove(Node node, Node root, float elapsed)
		{
			// Step 1:	Find new Xfrm
			//
			//			This is usually just a new position. However,
			//			we can handle new scaling and rotation as well.

			Xfrm desired = null;
			bool moved = GetDesiredPosition(current, elapsed, ref desired);
			//[Requirement("Operational.Minimum frame rate of 15 fps (#1305)")]
			//[Requirement("Operational.Maximum user input latency of 1 second (#1310)")]
			//[Requirement("Operational.Target frame rate of 60 fps (#1300)")]
			if (!moved || desired == null)
				return;

			// If this is the first update, don't bother checking for
			// collision
			//[Requirement("Operational.Minimum frame rate of 15 fps (#1305)")]
			//[Requirement("Operational.Maximum user input latency of 1 second (#1310)")]
			//[Requirement("Operational.Target frame rate of 60 fps (#1300)")]
			if (current.IsIdentity())
			{
				current = desired;
				return;
			}

			//[Feature("SceneGraph.CollisionDetection")]
			//[Requirement("App Development.Collision detection (#1150)")]
			{
				// If no collision listeners are present then don't bother
				// checking for collisions (similar to IgnoreCollision)
				//[Feature("SceneGraph.CollisionDetection")]
				//[Requirement("Operational.Minimum frame rate of 15 fps (#1305)")]
				//[Requirement("Operational.Maximum user input latency of 1 second (#1310)")]
				//[Requirement("Operational.Target frame rate of 60 fps (#1300)")]
				if (listeners.Count <= 0)
				{
					current = desired;
					return;
				}
	
	//			Debug.Assert(current != desired);
	
				// Step 2:	Detect collision
	
	            //[Requirement("Debugging.Exception/error detection (#625)")]
    	        //[Requirement("Debugging.Exception/error handling (#630)")]
				Debug.Assert(listeners.Count > 0 );
	
				Node nodeHit;
				IntersectInformation closestHit;
				if (IsCollsion(node, root, current, desired, out nodeHit, out closestHit))
				{
					// Step 3:	Find the collision point
					Xfrm collided;
					GetCollisionPosition(current, desired, closestHit, out collided);
	
					// Step 4:	Notify all the collision listeners
					for(int i = 0; i < listeners.Count; ++i)
					{
						ICollisionListener listener = (ICollisionListener) listeners[i];
						listener.OnCollided(	node,
												root,
												nodeHit,
												current,
												collided,
												closestHit,
												ref desired);
					}
				}
				else if (closestHit.Dist != float.PositiveInfinity)
				{
					// Step 4:	Notify all the collision listeners
					for(int i = 0; i < listeners.Count; ++i)
					{
						ICollisionListener listener = (ICollisionListener) listeners[i];
						listener.OnBeforeCollided(	node,
													root,
													nodeHit,
													current,
													closestHit,
													desired);
					}
				}
			}

			// Step 5:	Update the object position
			current = desired;
		}

		public virtual bool GetDesiredPosition(Xfrm cur, float elapsed, ref Xfrm desired)
		{
			//[Requirement("Operational.Minimum frame rate of 15 fps (#1305)")]
			//[Requirement("Operational.Maximum user input latency of 1 second (#1310)")]
			//[Requirement("Operational.Target frame rate of 60 fps (#1300)")]
			return false;
		}

		[Feature("SceneGraph.CollisionDetection")]
		[Requirement("App Development.Collision detection (#1150)")]
		[Requirement("Rendering.Scene graph (#735)")]
		public virtual void GetCollisionPosition(Xfrm cur, Xfrm desired, IntersectInformation closestHit, out Xfrm next)
		{
			Vector3 rayDir = 
				Goblin.Engine.Utilities.MyUtility.MatrixGetTranslation(desired.position) -
				Goblin.Engine.Utilities.MyUtility.MatrixGetTranslation(cur.position);
			rayDir.Normalize();
			rayDir.Scale(closestHit.Dist);

			next = cur.Copy();
			next.translate(rayDir);
		}

		[Feature("SceneGraph.CollisionDetection")]
		[Requirement("Rendering.Scene graph (#735)")]
		[Requirement("App Development.Collision detection (#1150)")]
		bool IsCollsion(Node node, Node root, Xfrm cur, Xfrm next, out Node nodeHit,
			out IntersectInformation closestHit)
		{
			if ( node.MyMeshCont == null ) 
			{
				nodeHit = null;
				closestHit = new IntersectInformation();
				return false;
			}

			Matrix curMat = cur.Combine();
			Matrix nextMat = next.Combine();

		//	Debug.Assert(curMat != nextMat);

			Bounds boundsNode = node.Bounds;

			Vector3[] curVerts;
			node.MyMeshCont.getTransformedVertices(curMat, out curVerts);

			Vector3[] newVerts;
			node.MyMeshCont.getTransformedVertices(nextMat, out newVerts);

			return Goblin.Engine.Physics.Collision.IsCollision(	node, 
																boundsNode,
																curVerts, 
																newVerts,
																root,
																!node.MyMeshCont.FineGrainCD,
																out nodeHit,
																out closestHit);
		}
	}
}
