///////////////////////////////////////////////////////////////////////
//  Copyright (C) 2005 Columbia University.  All Rights Reserved.
//
//      $Header: /Goblin/Engine/Scene/Transformers/VelocityTransformer.cs 6     1/12/07 2:23p Eaddy $
//
//    File name:	VelocityTransformer.cs
//   Created by:	Marc Eaddy
//   Created on:	04/18/2005
//
//      Purpose:	
//
//  Description:	
//
///////////////////////////////////////////////////////////////////////
/**
 * $Log: /Goblin/Engine/Scene/Transformers/VelocityTransformer.cs $
 * 
 * 6     1/12/07 2:23p Eaddy
 * o After annotating all statements with their corresponding requirements
 * 
 * 5     9/29/06 6:18p Eaddy
 * o Annotated program elements with the concern they realize
 * 
 * 4     5/28/05 1:21p Eaddy
 * Changed copyright
 * 
 * 3     5/17/05 3:32a Eaddy
 * o Trying to fix robot AI
 * 
 * 2     4/25/05 4:42p Eaddy
 * o Transformer no longer stores the Node. This eliminates an unnecessary
 * circular dependency
 * between Node and Transformer, allows Transformers to be reused, and
 * simplifies the design.
 * 
 * 1     4/22/05 4:59p Eaddy
 * o Broke out Transformer and CollisionListener classes into separate
 * files
 */

using Phx.Aop.Attributes;

using System;

using System.Diagnostics;			// Debug.Assert()

using Microsoft.DirectX;			// Matrix
using Microsoft.DirectX.Direct3D;	// IntersectionInformation

namespace Goblin.Engine.Scene
{
	/// <summary>
	/// Moves an object by moving it forward in the direction it is pointing
	/// </summary>
	[Feature("SceneGraph.Movement")]
	[Requirement("Rendering.Scene graph (#735)")]
	[Requirement("App Development.Simple physics (#1155)")]
	[Requirement("App Development.Collision detection (#1150)")]
	public class VelocityTransformer : Transformer, ICollisionListener
	{
		float velocity = 0;

		public VelocityTransformer()
		{ 
			//[Feature("SceneGraph.CollisionDetection")]
			//[Requirement("App Development.Collision detection (#1150)")]
			AddCollisionListener(this);
		}

		[Requirement("App Development.Object model (#1110)")]
		[Requirement("Rendering.Scene graph (#735)")]
		[Requirement("App Development.Simple physics (#1155)")]
		public float Velocity
		{
			get { return velocity; }
			set { velocity = value; }
		}

		public override bool GetDesiredPosition(Xfrm cur, float elapsed, ref Xfrm desired)
		{
			//[Requirement("Operational.Minimum frame rate of 15 fps (#1305)")]
			//[Requirement("Operational.Maximum user input latency of 1 second (#1310)")]
			//[Requirement("Operational.Target frame rate of 60 fps (#1300)")]
			if (velocity == 0)
				return false;

			Vector4 vecDir = new Vector4(0, 0, 1, 0);
			vecDir.Transform(cur.position);
			vecDir.Scale(velocity);
			Vector3 vecDir2 = new Vector3(vecDir.X, vecDir.Y, vecDir.Z);

			desired = cur.Copy();
			desired.position = cur.position * Matrix.Translation(vecDir2);

            //[Requirement("Debugging.Exception/error detection (#625)")]
            //[Requirement("Debugging.Exception/error handling (#630)")]
			Debug.Assert(cur.position != desired.position);

			//[Requirement("Operational.Minimum frame rate of 15 fps (#1305)")]
			//[Requirement("Operational.Maximum user input latency of 1 second (#1310)")]
			//[Requirement("Operational.Target frame rate of 60 fps (#1300)")]
			return true;
		}

		[Feature("SceneGraph.CollisionDetection")]
		[Requirement("App Development.Collision detection (#1150)")]
		public void OnBeforeCollided(	Node node,
										Node root,                      
										Node nodeHit,                   
										Xfrm old,                       
										IntersectInformation intersect, 
										Xfrm desired)                   
		{                                                               
			// Do nothing
		}                                                               

		[Feature("SceneGraph.CollisionDetection")]
		[Requirement("App Development.Collision detection (#1150)")]
		public void OnCollided(	Node node,
								Node root,                              
								Node nodeHit,                           
								Xfrm old,                               
								Xfrm collided,                          
								IntersectInformation intersect,         
								ref Xfrm desired)                       
		{                                                               
			velocity = 0;                                               
		}                                                               
	}                                                                   
}                                                                       
