///////////////////////////////////////////////////////////////////////
//  Copyright (C) 2005 Columbia University.  All Rights Reserved.
//
//      $Header: /Goblin/Engine/Scene/Transformers/Xfrm.cs 7     1/22/07 12:13a Eaddy $
//
//    File name:	Xfrm.cs
//   Created by:	Marc Eaddy
//   Created on:	04/18/2005
//
//      Purpose:	
//
//  Description:	
//
///////////////////////////////////////////////////////////////////////
/**
 * $Log: /Goblin/Engine/Scene/Transformers/Xfrm.cs $
 * 
 * 7     1/22/07 12:13a Eaddy
 * o Cleaned up some redundant and missing annotations
 * 
 * 6     1/12/07 2:23p Eaddy
 * o After annotating all statements with their corresponding requirements
 * 
 * 5     9/29/06 6:18p Eaddy
 * o Annotated program elements with the concern they realize
 * 
 * 4     6/02/05 9:23p Eaddy
 * o Fixed equality operators
 * o Added Multiply
 * 
 * 3     5/28/05 1:21p Eaddy
 * Changed copyright
 * 
 * 2     5/17/05 3:32a Eaddy
 * o Trying to fix robot AI
 * 
 * 1     4/22/05 4:59p Eaddy
 * o Broke out Transformer and CollisionListener classes into separate
 * files
 */

using Phx.Aop.Attributes;

using System;

using Microsoft.DirectX;			// Matrix

namespace Goblin.Engine.Scene
{
	[Feature("SceneGraph.Movement")]
	[Requirement("Rendering.Scene graph (#735)")]
	public class Xfrm : ICloneable
	{
		public Matrix scaling = Matrix.Identity;
		public Matrix rotation = Matrix.Identity;
		public Matrix position = Matrix.Identity;

		public override int GetHashCode()
		{
			return base.GetHashCode();
		}

		public override bool Equals(object rhs)
		{
			//[Requirement("Debugging.Exception/error detection (#625)")]
			if (rhs == null)
				//[Requirement("Debugging.Exception/error handling (#630)")]
				return false;

			Xfrm rhsXfrm = rhs as Xfrm;
			//[Requirement("Debugging.Exception/error detection (#625)")]
			if ((object) rhsXfrm == null)
				//[Requirement("Debugging.Exception/error handling (#630)")]
				return false;
			else
				return this == rhsXfrm; // Calls operator==
		}

		public static bool operator==(Xfrm lhs, Xfrm rhs)
		{
			if (Object.ReferenceEquals(lhs, rhs))
			{
				return true;
			}
			//:block[Requirement("Debugging.Exception/error detection (#625)")]
			else if ((object) lhs == null || (object) rhs == null)
			{
				//[Requirement("Debugging.Exception/error handling (#630)")]
				return false;
			}
			else
			{
				return lhs.position == rhs.position;// &&
				//lhs.rotation == rhs.rotation &&
				//lhs.scaling == rhs.scaling;
			}
		}

		public static bool operator!=(Xfrm lhs, Xfrm rhs)
		{
			return !(lhs == rhs);
		}

		public void Multiply(Xfrm rhs)
		{
			position *= rhs.position;
			scaling *= rhs.scaling;
			rotation *= rhs.rotation;
		}

		public bool IsIdentity()
		{
			return position == Matrix.Identity;// &&
			//rotation == Matrix.Identity &&
			//scaling == Matrix.Identity;
		}

		public Matrix Combine()
		{
			return scaling * rotation * position;
		}

		public object Clone()
		{
			Xfrm copy = new Xfrm();
			copy.scaling = scaling;
			copy.rotation = rotation;
			copy.position = position;
			return copy;
		}

		public Xfrm Copy()
		{
			Xfrm copy = new Xfrm();
			copy.scaling = scaling;
			copy.rotation = rotation;
			copy.position = position;
			return copy;
		}

		public void setPosition( float x, float y, float z ) 
		{
			position = Matrix.Translation( x, y, z );
		}

		public void translate( float x, float y, float z ) 
		{
			position *= Matrix.Translation( x, y, z );
		}

		public void translate( Vector3 vec ) 
		{
			position *= Matrix.Translation( vec );
		}

		public void rotateX( float radians )
		{
			rotation *= Matrix.RotationX( radians );
		}

		public void rotateY( float radians )
		{
			rotation *= Matrix.RotationY( radians );
		}

		public void rotateZ( float radians )
		{
			rotation *= Matrix.RotationZ( radians );
		}

		public void rotateAxis( Vector3 axis, float radians )
		{
			rotation *= Matrix.RotationAxis( axis, radians );
		}

		public void rotateYPR( float yaw, float pitch, float roll )
		{
			rotation *= Matrix.RotationYawPitchRoll( yaw, pitch, roll );
		}

		public void setScaling( float x, float y, float z ) 
		{
			scaling = Matrix.Scaling( x, y, z );
		}

		public Matrix Scale
		{
			get { return scaling; }
			set { scaling = value; }
		}

		public void scale( float scalar )
		{
			scale( scalar, scalar, scalar );
		}

		public void scale( float x, float y, float z )
		{
			scaling *= Matrix.Scaling( x, y, z );
		}
	}
}
