///////////////////////////////////////////////////////////////////////
//  Copyright (C) 2004 Columbia University.  All Rights Reserved.
//
//      $Header: /Goblin/Engine/Utilities/MyUtility.cs 17    2/02/07 5:18a Eaddy $
//
//    File name:	MyUtility.cs
//   Created By:	Marc Eaddy
//   Created on:	2/18/05
//
//      Purpose:	
//
//  Description:	
//
///////////////////////////////////////////////////////////////////////
/**
 * $Log: /Goblin/Engine/Utilities/MyUtility.cs $
 * 
 * 17    2/02/07 5:18a Eaddy
 * o Final assignment used for ASOC submission
 * 
 * 16    1/22/07 12:13a Eaddy
 * o Cleaned up some redundant and missing annotations
 * 
 * 15    1/19/07 5:32p Eaddy
 * o Added Requirement annotations
 * 
 * 14    1/12/07 2:24p Eaddy
 * o After annotating all statements with their corresponding requirements
 * 
 * 13    10/10/06 3:15p Eaddy
 * o Removed redundant concern annotations
 * o We now have 100% concern annotation coverage
 * 
 * 12    9/29/06 6:18p Eaddy
 * o Annotated program elements with the concern they realize
 * 
 * 11    5/02/06 11:03a Eaddy
 * o Moved MyTrace into separate library
 * 
 * 10    5/31/05 5:51p Eaddy
 * o Added MeshToString and can now handle position-only stream formats
 * 
 * 9     5/28/05 8:06p Eaddy
 * o Now all trace statements go thru MyTrace for consistency
 * 
 * 8     5/28/05 1:21p Eaddy
 * Changed copyright
 * 
 * 7     4/15/05 2:48p Eaddy
 * 
 * 6     4/05/05 6:39p Eaddy
 * o Moved code into helper class
 * 
 * 5     2/25/05 9:21p Eaddy
 * o Added TraceMesh() to output a mesh to debug output. This is also
 * useful for figuring out how to access
 * the vertices in a mesh.
 * 
 * 4     2/25/05 8:15p Eaddy
 * o Added Vector3ToString
 * 
 * 3     2/18/05 7:55p Eaddy
 * o Made method names consistent by starting them with uppercase
 * 
 * 2     2/18/05 7:47p Eaddy
 * o Added helper function
 * 
 * 1     2/18/05 7:36p Eaddy
 * o Initial checkin
 */

using Phx.Aop.Attributes;

using System;
using System.IO;
using System.Text;
using System.Diagnostics;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

using Microsoft.Samples.DirectX.UtilityToolkit;

using Columbia.Utilities;

namespace Goblin.Engine.Utilities
{
	/// <summary>
	/// Summary description for MyUtility.
	/// </summary>
	public class MyUtility
	{ 
		[Feature("EC")]
		[Requirement("Debugging.Exception/error detection (#625)")]
		const uint D3DXFERR_FILENOTFOUND = 0x88760389;

		[Feature("Persistence.Loading")]
		[Requirement("Configuration.Import - DirectX (#720)")]
		[Requirement("Configuration.Device configuration files (#135)")]
		[Requirement("Configuration.XML (#400)")]
		public static string MyFindMediaFile(string filename)
		{
			return MyFindMediaFile(null, filename);
		}

		[Feature("Persistence.Loading")]
		[Requirement("Configuration.Import - DirectX (#720)")]
		[Requirement("Configuration.Device configuration files (#135)")]
		[Requirement("Configuration.XML (#400)")]
		public static string MyFindMediaFile(string subdirectory, string filename)
		{
			string path = null;

			//[Feature("EC")]
            //[Requirement("Debugging.Exception/error detection (#625)")]
			try
			{
				//[Feature("Persistence.Loading")]
				string root = Path.GetPathRoot(filename);
				//[Feature("Persistence.Loading")]
				if (root.Length > 0)
				{
					path = filename;
				}
				else if (subdirectory != null && subdirectory.Length > 0)
				{
					path = Utility.FindMediaFile( Path.Combine(subdirectory, filename) );
				}
				else
				{
					path = Utility.FindMediaFile(filename);
				}
			}
            //:block[Feature("EH")]
			//:block[Requirement("Debugging.Exception/error handling (#630)")]
			catch (GraphicsException ex)
			{
				//[Requirement("Debugging.Exception/error detection (#625)")]
				if ((uint) ex.ErrorCode == D3DXFERR_FILENOTFOUND)
				{
					//[Feature("Logging")]
					//[Requirement("Debugging.Logging (#600)")]
					//[Requirement("Debugging.Exception/error detection (#625)")]
					MyTrace.WriteLine("X file not found: " + filename);
					//[Requirement("Debugging.Exception/error detection (#625)")]
					return null; // FILE NOT FOUND ERROR
				}
				else
				{
					//[Feature("Logging")]
					//[Requirement("Debugging.Logging (#600)")]
					//[Requirement("Debugging.Exception/error detection (#625)")]
					MyTrace.WriteLine(ex);
					//[Requirement("Debugging.Exception/error detection (#625)")]
					return null; // SOME OTHER ERROR
				}
			}
            //:block[Feature("EH")]
			//:block[Requirement("Debugging.Exception/error handling (#630)")]
			catch (MediaNotFoundException ex)
			{
				//[Feature("Logging")]
				//[Requirement("Debugging.Logging (#600)")]
				//[Requirement("Debugging.Exception/error detection (#625)")]
				MyTrace.WriteLine("Could not open X file: " + filename);
				//[Feature("Logging")]
				//[Requirement("Debugging.Logging (#600)")]
				//[Requirement("Debugging.Exception/error detection (#625)")]
				MyTrace.WriteLine(ex);
			}

			//[Requirement("Configuration.Import - DirectX (#720)")]
			//[Requirement("Configuration.Device configuration files (#135)")]
			//[Requirement("Configuration.XML (#400)")]
			//[Requirement("Debugging.Exception/error detection (#625)")]
			return path;
		}

		[Feature("Diagnostics")]
		[Requirement("Nonfunctional.Unused (#1400)")]
		public static string Vector3ToDXString(Vector3 v)
		{
			return string.Format("{0:F6};{1:F6};{2:F6};",
				v.X, v.Y, v.Z);
		}

		[Feature("Diagnostics")]
		[Requirement("Nonfunctional.Unused (#1400)")]
		public static string Vector3ToString(Vector3 v)
		{
			return string.Format("{0,7:F4},{1,7:F4},{2,7:F4}",
				v.X, v.Y, v.Z);
		}

		[Feature("Diagnostics")]
		[Requirement("Nonfunctional.Unused (#1400)")]
		public static string Vector3ToString(Vector3 v, string name)
		{
			return string.Format("{0}: {1}", name, Vector3ToString(v));     
		}

		[Feature("Diagnostics")]
		[Requirement("Debugging.Logging (#600)")]
		public static string MatrixToString(Matrix mat)
		{
			return                                                          
				string.Format(	"{0,7:F4},{1,7:F4},{2,7:F4},{3,7:F4}\n",
								mat.M11, mat.M12, mat.M13, mat.M14) +
				string.Format(	"{0,7:F4},{1,7:F4},{2,7:F4},{3,7:F4}\n",
								mat.M21, mat.M22, mat.M23, mat.M24) +
				string.Format(	"{0,7:F4},{1,7:F4},{2,7:F4},{3,7:F4}\n",
								mat.M31, mat.M32, mat.M33, mat.M34) +
				string.Format(	"{0,7:F4},{1,7:F4},{2,7:F4},{3,7:F4}\n",
								mat.M41, mat.M42, mat.M43, mat.M44);
		}

		[Feature("Diagnostics")]
		[Requirement("Debugging.Logging (#600)")]
		public static string MatrixToString(Matrix mat, string name)
		{
			return string.Format("{0}:\n", name) +
				MatrixToString(mat);
		}

        [Feature("SceneGraph")]
		[Requirement("Device.Single monitor displays (#165)")]
		[Requirement("Device.Multi-monitor displays (#190)")]
		[Requirement("Device.Head mounted displays (#195)")]
		[Requirement("3D UI.Optical see-through HMD support (#910)")]
		[Requirement("3D UI.Monoscopic display support (#915)")]
		[Requirement("Rendering.Multiple coordinate systems (#705)")]
		[Requirement("Rendering.Relative coordinate systems (#710)")]
		[Requirement("Rendering.Scene graph (#735)")]
		public static Vector3 MatrixGetTranslation(Matrix mat)
		{
			return new Vector3(mat.M41, mat.M42, mat.M43);
		}

		[Feature("Diagnostics")]
		[Requirement("Nonfunctional.Unused (#1400)")]
		public static string MeshToString(BaseMesh mesh, string name)
		{
			VertexBuffer vb = mesh.VertexBuffer;

            //[Feature("EC")]
            //[Requirement("Debugging.Exception/error detection (#625)")]
			if (vb.Description.VertexFormat != CustomVertex.PositionNormal.Format &&
				vb.Description.VertexFormat != CustomVertex.PositionOnly.Format)
			{
				//[Feature("Logging")]
				//[Requirement("Debugging.Exception/error handling (#630)")]
				//[Requirement("Debugging.Logging (#600)")]
				MyTrace.Log(TraceLevel.Info, "Don't know how to printout mesh format '{0}'.",
					name);
				//[Requirement("Debugging.Exception/error handling (#630)")]
				return "";
			}

			IndexBuffer ib = mesh.IndexBuffer;

			short[] indices = (short[]) ib.Lock(0,
				typeof(short), LockFlags.ReadOnly, mesh.NumberFaces*3);

			StringBuilder buf = new StringBuilder();

			buf.Append("  Mesh " + name + " {\r\n");
			buf.Append("   " + mesh.NumberVertices + ";\r\n");

			CustomVertex.PositionNormal[] verticesPN = null;
			CustomVertex.PositionOnly[] verticesP = null;

			if (vb.Description.VertexFormat == CustomVertex.PositionNormal.Format)
			{
				verticesPN = (CustomVertex.PositionNormal[]) vb.Lock(0,
					typeof(CustomVertex.PositionNormal),
					LockFlags.ReadOnly,
					mesh.NumberVertices);
			}
			else if (vb.Description.VertexFormat == CustomVertex.PositionOnly.Format)
			{
				verticesP = (CustomVertex.PositionOnly[]) vb.Lock(0,
					typeof(CustomVertex.PositionOnly),
					LockFlags.ReadOnly,
					mesh.NumberVertices);
			}

			//[Feature("EC")]
            //[Requirement("Debugging.Exception/error detection (#625)")]
            //[Requirement("Debugging.Exception/error handling (#630)")]
			Debug.Assert(verticesPN != null || verticesP != null);

			for(int i = 0; i < mesh.NumberVertices; ++i)
			{
				buf.Append("   ");

				if (verticesPN != null)
					buf.Append(MyUtility.Vector3ToDXString(verticesPN[i].Position));
				else if (verticesP != null)
					buf.Append(MyUtility.Vector3ToDXString(verticesP[i].Position));
			
				if (i != mesh.NumberVertices - 1)
					buf.Append(",\r\n");
				else
					buf.Append(";\r\n");
			}

			buf.Append("   " + mesh.NumberFaces + ";\r\n");

			for(int i = 0; i < mesh.NumberFaces; ++i)
			{
				short nIndexPt1 = indices[(i*3)+0];
				short nIndexPt2 = indices[(i*3)+1];
				short nIndexPt3 = indices[(i*3)+2];

				buf.AppendFormat("   3;{0},{1},{2};",
					nIndexPt1, nIndexPt2, nIndexPt3);

				if (i != mesh.NumberFaces - 1)
					buf.Append(",\r\n");
				else
					buf.Append(";\r\n");
			}

			//[Requirement("Nonfunctional.Clean shutdown (#1405)")]
			{
				vb.Unlock();
				vb.Dispose();
	
				ib.Unlock();
				ib.Dispose();
			}

			return buf.ToString();
		}

		[Feature("Diagnostics")]
		[Requirement("Debugging.Logging (#600)")]
		public static string XFileGuid2Str(System.Guid guid)
		{
			if (guid == XFileGuid.Animation)
				return "Animation";
			else if (guid == XFileGuid.AnimationKey)
				return "AnimationKey";
			else if (guid == XFileGuid.AnimationOptions)
				return "AnimationOptions";
			else if (guid == XFileGuid.AnimationSet)
				return "AnimationSet";
			else if (guid == XFileGuid.ApplicationData)
				return "ApplicationData";
			else if (guid == XFileGuid.Boolean)
				return "Boolean";
			else if (guid == XFileGuid.Boolean2d)
				return "Boolean2d";
			else if (guid == XFileGuid.Camera)
				return "Camera";
			else if (guid == XFileGuid.ColorRgb)
				return "ColorRgb";
			else if (guid == XFileGuid.ColorRgba)
				return "ColorRgba";
			else if (guid == XFileGuid.Coordinates2D)
				return "Coordinates2D";
			else if (guid == XFileGuid.ExternalVisual)
				return "ExternalVisual";
			else if (guid == XFileGuid.FloatKeys)
				return "FloatKeys";
			else if (guid == XFileGuid.Frame)
				return "Frame";
			else if (guid == XFileGuid.FramePosition)
				return "FramePosition";
			else if (guid == XFileGuid.FrameRotation)
				return "FrameRotation";
			else if (guid == XFileGuid.FrameTransformMatrix)
				return "FrameTransformMatrix";
			else if (guid == XFileGuid.FrameVelocity)
				return "FrameVelocity";
			else if (guid == XFileGuid.IndexedColor)
				return "IndexedColor";
			else if (guid == XFileGuid.Info)
				return "Info";
			else if (guid == XFileGuid.InlineData)
				return "InlineData";
			else if (guid == XFileGuid.Light)
				return "Light";
			else if (guid == XFileGuid.LightAttenuation)
				return "LightAttenuation";
			else if (guid == XFileGuid.LightPenumbra)
				return "LightPenumbra";
			else if (guid == XFileGuid.LightRange)
				return "LightRange";
			else if (guid == XFileGuid.LightUmbra)
				return "LightUmbra";
			else if (guid == XFileGuid.Material)
				return "Material";
			else if (guid == XFileGuid.MaterialAmbientColor)
				return "MaterialAmbientColor";
			else if (guid == XFileGuid.MaterialArray)
				return "MaterialArray";
			else if (guid == XFileGuid.MaterialDiffuseColor)
				return "MaterialDiffuseColor";
			else if (guid == XFileGuid.MaterialEmissiveColor)
				return "MaterialEmissiveColor";
			else if (guid == XFileGuid.MaterialPower)
				return "MaterialPower";
			else if (guid == XFileGuid.MaterialSpecularColor)
				return "MaterialSpecularColor";
			else if (guid == XFileGuid.MaterialWrap)
				return "MaterialWrap";
			else if (guid == XFileGuid.Matrix4x4)
				return "Matrix4x4";
			else if (guid == XFileGuid.Mesh)
				return "Mesh";
			else if (guid == XFileGuid.MeshFace)
				return "MeshFace";
			else if (guid == XFileGuid.MeshFaceWraps)
				return "MeshFaceWraps";
			else if (guid == XFileGuid.MeshMaterialList)
				return "MeshMaterialList";
			else if (guid == XFileGuid.MeshNormals)
				return "MeshNormals";
			else if (guid == XFileGuid.MeshTextureCoords)
				return "MeshTextureCoords";
			else if (guid == XFileGuid.MeshVertexColors)
				return "MeshVertexColors";
			else if (guid == XFileGuid.ProgressiveMesh)
				return "ProgressiveMesh";
			else if (guid == XFileGuid.PropertyBag)
				return "PropertyBag";
			else if (guid == XFileGuid.RightHanded)
				return "RightHanded";
			else if (guid == XFileGuid.RmGuid)
				return "RmGuid";
			else if (guid == XFileGuid.StringProperty)
				return "StringProperty";
			else if (guid == XFileGuid.TextureFilename)
				return "TextureFilename";
			else if (guid == XFileGuid.TextureReference)
				return "TextureReference";
			else if (guid == XFileGuid.TimedFloatKeys)
				return "TimedFloatKeys";
			else if (guid == XFileGuid.Url)
				return "Url";
			else if (guid == XFileGuid.Vector)
				return "Vector";
			//:block[Feature("EC")]
			//:block[Requirement("Debugging.Exception/error detection (#625)")]
			else
				return "<UNKNOWN: " + guid.ToString() + ">";
		}
	}                                                                                   
}
